@@ -51,6 +51,7 @@
 #include <feel/feelcrb/parameterspace.hpp>
 
 #include <feel/feelcrb/modelcrbbase.hpp>
+#include <feel/feeldiscr/reducedbasisspace.hpp>
 
 namespace Feel
 {
@@ -65,7 +66,7 @@ makeAdvectionDiffusionOptions()
     ( "mu2", po::value<double>()->default_value( 0.1 ), "Peclet number in [0.1;100]" )
     ( "no-export", "don't export results" )
     ;
-    return AdvectionDiffusionoptions.add( Feel::feel_options() );
+    return AdvectionDiffusionoptions;
 }
 AboutData
 makeAdvectionDiffusionAbout( std::string const& str = "AdvectionDiffusion" )
@@ -104,6 +105,10 @@ public :
 
     /*space*/
     typedef FunctionSpace<mesh_type, basis_type, value_type> space_type;
+
+    static const bool is_time_dependent = false;
+    static const bool is_linear = true;
+
 };
 /**
  * \class AdvectionDiffusion
@@ -142,7 +147,8 @@ public :
  * @author Christophe Prud'homme
  * @see
  */
-    class AdvectionDiffusion : public ModelCrbBase<ParameterDefinition,FunctionSpaceDefinition>
+class AdvectionDiffusion : public ModelCrbBase<ParameterDefinition,FunctionSpaceDefinition>,
+                           public boost::enable_shared_from_this< AdvectionDiffusion >
 {
 public:
 
@@ -156,7 +162,6 @@ public:
 
     static const uint16_type Order = 5;
     static const uint16_type ParameterSpaceDimension = 2;
-    static const bool is_time_dependent = false;
     //@}
 
     /** @name Typedefs
@@ -195,6 +200,10 @@ public:
     typedef space_type::element_type element_type;
     typedef boost::shared_ptr<element_type> element_ptrtype;
 
+    /*reduced basis space*/
+    typedef ReducedBasisSpace<super_type, mesh_type, basis_type, value_type> rbfunctionspace_type;
+    typedef boost::shared_ptr< rbfunctionspace_type > rbfunctionspace_ptrtype;
+
     /* export */
     typedef Exporter<mesh_type> export_type;
     typedef boost::shared_ptr<export_type> export_ptrtype;
@@ -299,6 +308,14 @@ public:
         return Xh;
     }
 
+    /**
+     * \brief Returns the reduced basis function space
+     */
+    rbfunctionspace_ptrtype rBFunctionSpace()
+    {
+        return RbXh;
+    }
+
     //! return the parameter space
     parameterspace_ptrtype parameterSpace() const
     {
@@ -309,14 +326,9 @@ public:
      * \brief compute the beta coefficient for both bilinear and linear form
      * \param mu parameter to evaluate the coefficients
      */
-    boost::tuple<beta_vector_type, std::vector<beta_vector_type> >
-    computeBetaQm( element_type const& T,parameter_type const& mu , double time=1e30 )
-    {
-        return computeBetaQm( mu , time );
-    }
 
     boost::tuple<beta_vector_type, std::vector<beta_vector_type> >
-    computeBetaQm( parameter_type const& mu , double time=0 )
+    computeBetaQm( parameter_type const& mu )
     {
         M_betaAqm.resize( Qa() );
         for(int i=0; i<Qa(); i++)
@@ -406,7 +418,7 @@ public:
     /**
      * H1 scalar product
      */
-    sparse_matrix_ptrtype innerProduct ( void )
+    sparse_matrix_ptrtype energyMatrix ( void )
     {
         return M;
     }
@@ -480,6 +492,7 @@ private:
 
     mesh_ptrtype mesh;
     space_ptrtype Xh;
+    rbfunctionspace_ptrtype RbXh;
     sparse_matrix_ptrtype D,M;
     vector_ptrtype F;
     element_ptrtype pT;
@@ -530,6 +543,7 @@ AdvectionDiffusion::initModel()
      * The function space and some associate elements are then defined
      */
     Xh = space_type::New( mesh );
+    RbXh = rbfunctionspace_type::New( this->shared_from_this() , mesh );
     // allocate an element of Xh
     pT = element_ptrtype( new element_type( Xh ) );
 
