@@ -519,34 +519,35 @@ OpusData::createMesh( double h, bool ref )
             << "/**\n"
             << " * Geometry for the Opus model\n"
             << " */\n"
-            << "p1=newp;Point(p1) = {0,0,0,h};\n"
-            << "p2=newp;Point(p2) = {e_PCB,0,0,h};\n"
+            << "hs = "<<h<<";\n"
+            << "p1=newp;Point(p1) = {0,0,0,hs};\n"
+            << "p2=newp;Point(p2) = {e_PCB,0,0,hs};\n"
             << "\n"
-            << "p3=newp;Point(p3) = {e_PCB,h_1,0,h};\n"
-            << "p4=newp;Point(p4) = {e_PCB,h_1+L_IC,0,h};\n"
+            << "p3=newp;Point(p3) = {e_PCB,h_1,0,hs};\n"
+            << "p4=newp;Point(p4) = {e_PCB,h_1+L_IC,0,hs};\n"
             << "\n"
             << "p5=newp;Point(p5) = {e_PCB,h_2,0,h};\n"
-            << "p6=newp;Point(p6) = {e_PCB,h_2+L_IC,0,h};\n"
+            << "p6=newp;Point(p6) = {e_PCB,h_2+L_IC,0,hs};\n"
             << "\n"
-            << "p7=newp;Point(p7) = {e_PCB,h_PCB,0,h};\n"
-            << "p8=newp;Point(p8) = {0,h_PCB,0,h};\n"
+            << "p7=newp;Point(p7) = {e_PCB,h_PCB,0,hs};\n"
+            << "p8=newp;Point(p8) = {0,h_PCB,0,hs};\n"
             << "\n"
             << "air_p1=p2;\n"
-            << "air_p21=newp;Point(air_p21) = {e_PCB+e_IC,0,0,h};\n"
-            << "air_p22=newp;Point(air_p22) = {e_PCB+e_A,0,0,h};\n"
-            << "air_p31=newp;Point(air_p31) = {e_PCB+e_A,h_PCB,0,h};\n"
-            << "air_p32=newp;Point(air_p32) = {e_PCB+e_IC,h_PCB,0,h};\n"
+            << "air_p21=newp;Point(air_p21) = {e_PCB+e_IC,0,0,hs};\n"
+            << "air_p22=newp;Point(air_p22) = {e_PCB+e_A,0,0,hs};\n"
+            << "air_p31=newp;Point(air_p31) = {e_PCB+e_A,h_PCB,0,hs};\n"
+            << "air_p32=newp;Point(air_p32) = {e_PCB+e_IC,h_PCB,0,hs};\n"
             << "air_p4=p7;\n"
             << "\n"
             << "air_p5=p3;\n"
-            << "air_p51=newp;Point(air_p51) = {e_PCB+e_IC,h_1,0,h};\n"
+            << "air_p51=newp;Point(air_p51) = {e_PCB+e_IC,h_1,0,hs};\n"
             << "air_p6=p4;\n"
-            << "air_p61=newp;Point(air_p61) = {e_PCB+e_IC,h_1+L_IC,0,h};\n"
+            << "air_p61=newp;Point(air_p61) = {e_PCB+e_IC,h_1+L_IC,0,hs};\n"
             << "\n"
             << "air_p7=p5;\n"
-            << "air_p71=newp;Point(air_p71) = {e_PCB+e_IC,h_2,0,h};\n"
+            << "air_p71=newp;Point(air_p71) = {e_PCB+e_IC,h_2,0,hs};\n"
             << "air_p8=p6;\n"
-            << "air_p81=newp;Point(air_p81) = {e_PCB+e_IC,h_2+L_IC,0,h};\n"
+            << "air_p81=newp;Point(air_p81) = {e_PCB+e_IC,h_2+L_IC,0,hs};\n"
             << "\n"
             << "ic1_p1=p3;\n"
             << "ic1_p2=air_p51;\n"
