@@ -26,11 +26,12 @@
    \author Christophe Prud'homme <christophe.prudhomme@feelpp.org>
    \date 2007-12-23
  */
-#ifndef __Backend_H
-#define __Backend_H 1
+#ifndef Backend_H
+#define Backend_H 1
 
 #include <boost/timer.hpp>
 #include <boost/tuple/tuple.hpp>
+#include <boost/tuple/tuple_comparison.hpp>
 #include <boost/fusion/include/fold.hpp>
 #include <boost/smart_ptr/enable_shared_from_this.hpp>
 
@@ -42,9 +43,10 @@
 #include <feel/feelalg/vector.hpp>
 #include <feel/feelalg/matrixsparse.hpp>
 #include <feel/feelalg/matrixblock.hpp>
-#include <feel/feelalg/vectorblock.hpp>
+//#include <feel/feelalg/vectorblock.hpp>
 #include <feel/feelalg/datamap.hpp>
 
+#include <feel/feelalg/solverlinear.hpp>
 #include <feel/feelalg/solvernonlinear.hpp>
 #include <feel/feelalg/preconditioner.hpp>
 #include <feel/feeldiscr/functionspacebase.hpp>
@@ -89,7 +91,7 @@ boost::shared_ptr<DataMap> datamap( T const& t, mpl::false_ )
 template<typename T>
 boost::shared_ptr<DataMap> datamap( T const& t )
 {
-    return datamap( t, detail::is_shared_ptr<T>() );
+    return datamap( t, Feel::detail::is_shared_ptr<T>() );
 }
 
 template<typename T>
@@ -108,9 +110,9 @@ T& ref( T& t, mpl::false_ )
     return t;
 }
 template<typename T>
-auto ref( T& t ) -> decltype( ref( t, detail::is_shared_ptr<T>() ) )
+auto ref( T& t ) -> decltype( ref( t, Feel::detail::is_shared_ptr<T>() ) )
 {
-    return ref( t, detail::is_shared_ptr<T>() );
+    return ref( t, Feel::detail::is_shared_ptr<T>() );
 }
 
 
@@ -122,6 +124,8 @@ template<int NR, int NC, typename T> class MatrixBlock;
 template<typename T> class VectorBlockBase;
 template<int NR, typename T> class VectorBlock;
 
+template<typename T> class BlocksBaseSparseMatrix;
+
 /**
  * \class Backend
  * \brief base class for all linear algebra backends
@@ -159,12 +163,15 @@ public:
     typedef SolverNonLinear<value_type> solvernonlinear_type;
     typedef boost::shared_ptr<solvernonlinear_type> solvernonlinear_ptrtype;
 
-    typedef boost::tuple<bool, size_type, value_type> solve_return_type;
-    typedef boost::tuple<bool, size_type, value_type> nl_solve_return_type;
+    typedef typename SolverLinear<value_type>::solve_return_type solve_return_type;
+    typedef typename solvernonlinear_type::solve_return_type nl_solve_return_type;
 
     typedef DataMap datamap_type;
     typedef boost::shared_ptr<datamap_type> datamap_ptrtype;
 
+    typedef typename datamap_type::indexsplit_type indexsplit_type;
+    typedef typename datamap_type::indexsplit_ptrtype indexsplit_ptrtype;
+
     //@}
 
     /** @name Constructors, destructor
@@ -224,7 +231,7 @@ public:
                                      const size_type m_l,
                                      const size_type n_l,
                                      graph_ptrtype const & graph,
-                                     std::vector < std::vector<size_type> > indexSplit,
+                                     indexsplit_ptrtype const& indexSplit,
                                      size_type matrix_properties = NON_HERMITIAN )
     {
         auto mat = this->newMatrix( m,n,m_l,n_l,graph,matrix_properties );
@@ -290,10 +297,16 @@ public:
                                        ( buildGraphWithTranspose, ( bool ),false )
                                        ( pattern_block,    *, ( BlocksStencilPattern(1,1,size_type( Pattern::HAS_NO_BLOCK_PATTERN ) ) ) )
                                        ( diag_is_nonzero,  *( boost::is_integral<mpl::_> ), true )
-                                       ( verbose,( int ),0 )
+                                       ( verbose,   ( bool ), option(_prefix=this->prefix(),_name="backend.verbose").template as<bool>() )
                                        ( collect_garbage, *( boost::is_integral<mpl::_> ), true )
                                      ) )
     {
+        if ( verbose )
+        {
+            Environment::logMemoryUsage( "backend::newMatrix begin" );
+        }
+
+        if ( !this->comm().isActive() ) return sparse_matrix_ptrtype();
 
         //auto mat = this->newMatrix( trial->map(), test->map(), properties, false );
         auto mat = this->newMatrix( trial->dofOnOff(), test->dofOn(), properties, false );
@@ -338,6 +351,10 @@ public:
 
         mat->zero();
         mat->setIndexSplit( trial->dofIndexSplit() );
+        if ( verbose )
+        {
+            Environment::logMemoryUsage( "backend::newMatrix end" );
+        }
         return mat;
     }
 
@@ -352,21 +369,45 @@ public:
     /**
      * instantiate a new block matrix sparse
      */
-    sparse_matrix_ptrtype newBlockMatrixImpl( vf::BlocksBase<sparse_matrix_ptrtype> const & b,
-                                              bool copy_values=true,
-                                              bool diag_is_nonzero=true )
-    {
-        typedef MatrixBlockBase<typename sparse_matrix_ptrtype::element_type::value_type> matrix_block_type;
-        boost::shared_ptr<matrix_block_type> mb( new matrix_block_type( b, *this, copy_values, diag_is_nonzero ) );
-        return mb->getSparseMatrix();
-    }
-
-    sparse_matrix_ptrtype newBlockMatrixImpl( vf::BlocksBase<boost::shared_ptr<GraphCSR> > const & b,
+    sparse_matrix_ptrtype newBlockMatrixImpl( BlocksBaseSparseMatrix<value_type> const & b,
                                               bool copy_values=true,
                                               bool diag_is_nonzero=true )
     {
         typedef MatrixBlockBase<value_type> matrix_block_type;
-        boost::shared_ptr<matrix_block_type> mb( new matrix_block_type( b, *this, diag_is_nonzero ) );
+        typedef boost::shared_ptr<matrix_block_type> matrix_block_ptrtype;
+
+        matrix_block_ptrtype mb;
+        if ( b.isClosed() )
+        {
+            mb.reset( new matrix_block_type( b, *this, copy_values, diag_is_nonzero ) );
+        }
+        else
+        {
+            BlocksBaseSparseMatrix<value_type> copyBlock( b );
+            copyBlock.close();
+            mb.reset( new matrix_block_type( copyBlock, *this, copy_values, diag_is_nonzero ) );
+        }
+        return mb->getSparseMatrix();
+    }
+
+    sparse_matrix_ptrtype newBlockMatrixImpl( BlocksBaseGraphCSR const & b,
+                                              bool copy_values=true,
+                                              bool diag_is_nonzero=true )
+    {
+        typedef MatrixBlockBase<value_type> matrix_block_type;
+        typedef boost::shared_ptr<matrix_block_type> matrix_block_ptrtype;
+
+        matrix_block_ptrtype mb;
+        if ( b.isClosed() )
+        {
+            mb.reset( new matrix_block_type( b, *this, diag_is_nonzero ) );
+        }
+        else
+        {
+            BlocksBaseGraphCSR copyBlock( b );
+            copyBlock.close();
+            mb.reset( new matrix_block_type( copyBlock, *this, diag_is_nonzero ) );
+        }
         return mb->getSparseMatrix();
     }
 
@@ -454,6 +495,8 @@ public:
                                      )
                                    )
     {
+        if ( !this->comm().isActive() ) return vector_ptrtype();
+
         return this->newVector( test->dof() );
     }
 
@@ -486,6 +529,14 @@ public:
         return M_ksp;
     }
 
+    /**
+     * \return the type of non linear solver
+     */
+    std::string snesType() const
+    {
+        return M_snesType;
+    }
+
     /**
      * \return the type of preconditioner
      */
@@ -520,6 +571,11 @@ public:
      **/
     SolverType kspEnumType() const;
 
+    /**
+     * \return enum snes solver type from string
+     */
+    SolverNonLinearType snesEnumType() const;
+
     /**
      * \return enum fieldsplit type from options
      **/
@@ -599,16 +655,33 @@ public:
      */
     size_type maxIterations() const
     {
-        return M_maxit;
+        return M_maxitKSP;
+    }
+    size_type maxIterationsKSP() const
+    {
+        return M_maxitKSP;
+    }
+    size_type maxIterationsKSPinSNES() const
+    {
+        return M_maxitKSPinSNES;
     }
-
-    /**
-     * \return the maximum number of SNES iterations
-     */
     size_type maxIterationsSNES() const
     {
         return M_maxitSNES;
     }
+    size_type maxIterationsKSPReuse() const
+    {
+        return M_maxitKSPReuse;
+    }
+    size_type maxIterationsKSPinSNESReuse() const
+    {
+        return M_maxitKSPinSNESReuse;
+    }
+    size_type maxIterationsSNESReuse() const
+    {
+        return M_maxitSNESReuse;
+    }
+
 
     /**
      * \return the KSP relative tolerance in SNES
@@ -680,7 +753,7 @@ public:
         M_rtolerance = rtolerance;
         M_dtolerance = dtolerance;
         M_atolerance = atolerance;
-        M_maxit = maxit;
+        M_maxitKSP = maxit;
     }
 
     BOOST_PARAMETER_MEMBER_FUNCTION( ( void ),
@@ -821,7 +894,7 @@ public:
                                        //(prec,(sparse_matrix_ptrtype), matrix )
                                        ( prec,( preconditioner_ptrtype ), preconditioner( _prefix=this->prefix(),_matrix=matrix,_pc=this->pcEnumType()/*LU_PRECOND*/,
                                                                                           _pcfactormatsolverpackage=this->matSolverPackageEnumType(), _backend=this->shared_from_this() ) )
-                                       ( maxit,( size_type ), M_maxit/*1000*/ )
+                                       ( maxit,( size_type ), M_maxitKSP/*1000*/ )
                                        ( rtolerance,( double ), M_rtolerance/*1e-13*/ )
                                        ( atolerance,( double ), M_atolerance/*1e-50*/ )
                                        ( dtolerance,( double ), M_dtolerance/*1e5*/ )
@@ -831,9 +904,14 @@ public:
                                        ( pc,( std::string ),M_pc/*"lu"*/ )
                                        ( ksp,( std::string ),M_ksp/*"gmres"*/ )
                                        ( pcfactormatsolverpackage,( std::string ), M_pcFactorMatSolverPackage )
+                                       ( verbose,   ( bool ), option(_prefix=this->prefix(),_name="backend.verbose").template as<bool>() )
                                      )
                                    )
     {
+        if ( verbose )
+        {
+            Environment::logMemoryUsage( "backend::solve begin" );
+        }
         this->setTolerances( _dtolerance=dtolerance,
                              _rtolerance=rtolerance,
                              _atolerance=atolerance,
@@ -870,7 +948,12 @@ public:
 
         //new
         _sol->close();
-        detail::ref( solution ) = *_sol;
+        Feel::detail::ref( solution ) = *_sol;
+        Feel::detail::ref( solution ).close();
+        if ( verbose )
+        {
+            Environment::logMemoryUsage( "backend::solve end" );
+        }
         return ret;
     }
 
@@ -942,25 +1025,30 @@ public:
                                        ( pc,( std::string ),M_pc/*"lu"*/ )
                                        ( ksp,( std::string ),M_ksp/*"gmres"*/ )
                                        ( pcfactormatsolverpackage,( std::string ), M_pcFactorMatSolverPackage )
+                                       ( verbose,   ( bool ), option(_prefix=this->prefix(),_name="backend.verbose").template as<bool>() )
                                      )
                                    )
     {
+        if ( verbose )
+        {
+            Environment::logMemoryUsage( "backend::nlSolve begin" );
+        }
         this->setTolerancesSNES( _stolerance=stolerance,
                                  _rtolerance=rtolerance,
                                  _atolerance=atolerance,
                                  _maxit=maxit );
         this->setSolverType( _pc=pc, _ksp=ksp,
                              _pcfactormatsolverpackage = pcfactormatsolverpackage );
-        vector_ptrtype _sol( this->newVector( detail::datamap( solution ) ) );
+        vector_ptrtype _sol( this->newVector( Feel::detail::datamap( solution ) ) );
         // initialize
-        *_sol = detail::ref( solution );
+        *_sol = Feel::detail::ref( solution );
         this->setTranspose( transpose );
         solve_return_type ret;
 
         // this is done with nonlinerarsolver
         if ( !residual )
         {
-            residual = this->newVector( ( detail::datamap( solution ) ) );
+            residual = this->newVector( ( Feel::detail::datamap( solution ) ) );
             //this->nlSolver()->residual( _sol, residual );
         }
 
@@ -979,8 +1067,12 @@ public:
 
         //new
         _sol->close();
-        detail::ref( solution ) = *_sol;
-        detail::ref( solution ).close();
+        Feel::detail::ref( solution ) = *_sol;
+        Feel::detail::ref( solution ).close();
+        if ( verbose )
+        {
+            Environment::logMemoryUsage( "backend::nlSolve end" );
+        }
         return ret;
     }
 
@@ -1087,10 +1179,12 @@ private:
     bool   M_reuseFailed;
     boost::timer M_timer;
     bool   M_transpose;
-    size_type    M_maxit, M_maxitSNES;
+    size_type    M_maxitKSP, M_maxitKSPinSNES, M_maxitSNES;
+    size_type    M_maxitKSPReuse, M_maxitKSPinSNESReuse, M_maxitSNESReuse;
     size_type    M_iteration;
     std::string M_export;
     std::string M_ksp;
+    std::string M_snesType;
     std::string M_pc;
     std::string M_fieldSplit;
     std::string M_pcFactorMatSolverPackage;
@@ -1110,12 +1204,12 @@ typedef boost::shared_ptr<backend_type> backend_ptrtype;
 namespace detail
 {
 class BackendManagerImpl:
-    public std::map<std::pair<BackendType,std::string>, backend_ptrtype >,
+    public std::map<boost::tuple<BackendType,std::string,int>, backend_ptrtype >,
     public boost::noncopyable
 {
 public:
     typedef backend_ptrtype value_type;
-    typedef std::pair<BackendType,std::string> key_type;
+    typedef boost::tuple<BackendType,std::string,int> key_type;
     typedef std::map<key_type, value_type> backend_manager_type;
 
 };
@@ -1125,8 +1219,8 @@ struct BackendManagerDeleterImpl
 {
     void operator()() const
         {
-            VLOG(2) << "[BackendManagerDeleter] clear BackendManager Singleton: " << detail::BackendManager::instance().size() << "\n";
-            detail::BackendManager::instance().clear();
+            VLOG(2) << "[BackendManagerDeleter] clear BackendManager Singleton: " << Feel::detail::BackendManager::instance().size() << "\n";
+            Feel::detail::BackendManager::instance().clear();
             VLOG(2) << "[BackendManagerDeleter] clear BackendManager done\n";
         }
 };
@@ -1143,6 +1237,7 @@ BOOST_PARAMETER_FUNCTION(
       ( name,           ( std::string ), "" )
       ( kind,           ( BackendType ), BACKEND_PETSC )
       ( rebuild,        ( bool ), false )
+      ( worldcomm,      (WorldComm), Environment::worldComm() )
     ) )
 {
     // register the BackendManager into Feel::Environment so that it gets the
@@ -1150,41 +1245,41 @@ BOOST_PARAMETER_FUNCTION(
     static bool observed=false;
     if ( !observed )
     {
-        Environment::addDeleteObserver( detail::BackendManagerDeleter::instance() );
+        Environment::addDeleteObserver( Feel::detail::BackendManagerDeleter::instance() );
         observed = true;
     }
 
 
     Feel::detail::ignore_unused_variable_warning( args );
 
-    auto git = detail::BackendManager::instance().find( std::make_pair( kind, name ) );
+    auto git = Feel::detail::BackendManager::instance().find( boost::make_tuple( kind, name, worldcomm.globalSize() ) );
 
-    if (  git != detail::BackendManager::instance().end() && ( rebuild == false ) )
+    if (  git != Feel::detail::BackendManager::instance().end() && ( rebuild == false ) )
     {
-        VLOG(2) << "[backend] found backend name=" << name << " kind=" << kind << " rebuild=" << rebuild << "\n";
+        VLOG(2) << "[backend] found backend name=" << name << " kind=" << kind << " rebuild=" << rebuild << " worldcomm.globalSize()=" << worldcomm.globalSize() << "\n";
         return git->second;
     }
 
     else
     {
-        if (  git != detail::BackendManager::instance().end() && ( rebuild == true ) )
+        if (  git != Feel::detail::BackendManager::instance().end() && ( rebuild == true ) )
             git->second->clear();
 
-        VLOG(2) << "[backend] building backend name=" << name << " kind=" << kind << " rebuild=" << rebuild << "\n";
+        VLOG(2) << "[backend] building backend name=" << name << " kind=" << kind << " rebuild=" << rebuild << " worldcomm.globalSize()=" << worldcomm.globalSize() << "\n";
 
         backend_ptrtype b;
         if ( vm.empty() )
         {
-            b = Feel::backend_type::build( kind );
+            b = Feel::backend_type::build( kind, worldcomm );
         }
         else
-            b = Feel::backend_type::build( vm, name );
+            b = Feel::backend_type::build( vm, name, worldcomm );
         VLOG(2) << "storing backend in singleton" << "\n";
-        detail::BackendManager::instance().operator[]( std::make_pair( kind, name ) ) = b;
+        Feel::detail::BackendManager::instance().operator[]( boost::make_tuple( kind, name, worldcomm.globalSize() ) ) = b;
         return b;
     }
 
 }
 
 }
-#endif /* __Backend_H */
+#endif /* Backend_H */
