@@ -31,6 +31,7 @@
 #include <feel/feelcore/feelpetsc.hpp>
 #include <feel/feelalg/vectorpetsc.hpp>
 #include <feel/feelalg/matrixpetsc.hpp>
+#include <feel/feelalg/functionspetsc.hpp>
 
 #if defined( FEELPP_HAS_PETSC_H )
 
@@ -46,8 +47,9 @@ namespace Feel
 
 template <typename T>
 inline
-MatrixPetsc<T>::MatrixPetsc()
+MatrixPetsc<T>::MatrixPetsc( WorldComm const& worldComm )
     :
+    super( worldComm ),
     M_destroy_mat_on_exit( true )
 {}
 
@@ -269,7 +271,7 @@ void MatrixPetsc<T>::init ( const size_type m,
                             const size_type n_l,
                             graph_ptrtype const& graph )
 {
-    //std::cout << "\n SEQUENTIAL : init with graph"<<std::endl;
+    VLOG(1) << "MatrixPetsc init with graph";
     this->setGraph( graph );
 
     {
@@ -284,16 +286,17 @@ void MatrixPetsc<T>::init ( const size_type m,
 
     MPI_Comm_rank ( this->comm(), &proc_id );
 
-    DVLOG(2) << "[MatrixPETSc::init()] m   = " << m << "\n";
-    DVLOG(2) << "[MatrixPETSc::init()] n   = " << n << "\n";
-    DVLOG(2) << "[MatrixPETSc::init()] m_l = " << m_l << "\n";
-    DVLOG(2) << "[MatrixPETSc::init()] n_l = " << n_l << "\n";
+    DVLOG(1) << "[MatrixPETSc::init()] proc_id   = " << proc_id << "\n";
+    DVLOG(1) << "[MatrixPETSc::init()] m   = " << m << "\n";
+    DVLOG(1) << "[MatrixPETSc::init()] n   = " << n << "\n";
+    DVLOG(1) << "[MatrixPETSc::init()] m_l = " << m_l << "\n";
+    DVLOG(1) << "[MatrixPETSc::init()] n_l = " << n_l << "\n";
 
     // Make sure the sparsity pattern isn't empty
     //FEELPP_ASSERT ( this->graph()->size() == n_l )( this->graph()->size() )( n_l ).warn( "incompatible diagonal non zero pattern" );
-    DVLOG(2) << "[MatrixPETSc::init()] graph size   = " << this->graph()->size() << "\n";
-    DVLOG(2) << "[MatrixPETSc::init()] graph first row entry on proc   = " << this->graph()->firstRowEntryOnProc() << "\n";
-    DVLOG(2) << "[MatrixPETSc::init()] graph last row entry on proc   = " << this->graph()->lastRowEntryOnProc() << "\n";
+    DVLOG(1) << "[MatrixPETSc::init()] graph size   = " << this->graph()->size() << "\n";
+    DVLOG(1) << "[MatrixPETSc::init()] graph first row entry on proc   = " << this->graph()->firstRowEntryOnProc() << "\n";
+    DVLOG(1) << "[MatrixPETSc::init()] graph last row entry on proc   = " << this->graph()->lastRowEntryOnProc() << "\n";
 
     if ( m==0 )
         return;
@@ -436,61 +439,20 @@ void MatrixPetsc<T>::init ( const size_type m,
 
 
 template <typename T>
-void MatrixPetsc<T>::setIndexSplit( std::vector< std::vector<size_type> > const &indexSplit )
+void MatrixPetsc<T>::setIndexSplit( indexsplit_ptrtype const& indexSplit )
 {
-    this->M_IndexSplit=indexSplit;
+    //this->M_IndexSplit=indexSplit;
+    super::setIndexSplit( indexSplit );
 
-    M_petscIS.resize( indexSplit.size() );
+    PetscConvertIndexSplit( M_petscIS, *indexSplit, this->comm() );
+    DVLOG(1) << "setIndexSplit done\n";
+}
 
-    int ierr=0;
-
-    for ( uint i = 0 ; i < indexSplit.size(); ++i )
-    {
-        PetscInt nDofForThisField = indexSplit[i].size();
-        //std::cout << "\n setIndexSplit " << i << " ndof:" << nDofForThisField << "\n";
-
-        PetscInt * petscSplit = new PetscInt[nDofForThisField];
-        std::copy( this->M_IndexSplit[i].begin(),
-                   this->M_IndexSplit[i].end(),
-                   petscSplit );
-
-
-
-#if (PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 2)
-        ierr = ISCreateGeneral( this->comm(),nDofForThisField,petscSplit/*this->M_IndexSplit[i].data()*/,PETSC_COPY_VALUES,&M_petscIS[i] );
-#else
-        ierr = ISCreateGeneral( this->comm(),nDofForThisField,petscSplit/*this->M_IndexSplit[i].data()*/,&M_petscIS[i] );
-#endif
-        CHKERRABORT( this->comm(),ierr );
-
-        delete[] petscSplit;
-
-#if 0
-        ISView( M_petscIS[i],PETSC_VIEWER_STDOUT_WORLD ); // PETSC_VIEWER_STDOUT_SELF
-
-        PetscInt n;
-        /*
-          Get the number of indices in the set
-        */
-        ISGetLocalSize( M_petscIS[i],&n );
-        std::cout << "Local size: " << n << "\n";
-        const PetscInt *nindices;
-
-        /*
-          Get the indices in the index set
-        */
-        ISGetIndices( M_petscIS[i],&nindices );
-
-        for ( int j = 0; j < n; ++j )
-        {
-            std::cout << nindices[j] << " ";
-        }
-
-        std::cout << "\n";
-#endif
-    }
-
-    //std::cout << "\n setIndexSplit done\n";
+template <typename T>
+void MatrixPetsc<T>::updatePCFieldSplit( PC & pc, indexsplit_ptrtype const& is )
+{
+    PetscConvertIndexSplit( M_petscIS, *is, this->comm() );
+    updatePCFieldSplit( pc );
 }
 
 template <typename T>
@@ -517,6 +479,8 @@ void MatrixPetsc<T>::updatePCFieldSplit( PC & pc )
 
         if ( std::string( PCFIELDSPLIT ) == std::string( pcName ) )
         {
+
+
             //std::cout << "\n updatePCFieldSplit " << M_petscIS.size() << "\n";
             M_mapPC[&pc]=true;
 
@@ -536,7 +500,19 @@ void MatrixPetsc<T>::updatePCFieldSplit( PC & pc )
 
                 CHKERRABORT( this->comm(),ierr );
             }
+#if 0
+            std::cout << " matrix petsc\n";
+            ierr = PCFieldSplitSetBlockSize(pc,3);
+            //CHKERRABORT( this->comm(),ierr );
+            const PetscInt ufields[] = {0,2},pfields[] = {1};
+            ierr = PCFieldSplitSetFields( pc ,"u", 2, ufields,ufields);
+            CHKERRABORT( this->comm(),ierr );
+            ierr = PCFieldSplitSetFields( pc , "p", 1, pfields,pfields);
+            CHKERRABORT( this->comm(),ierr );
+#endif
+
         }
+
     }
 
 #else
@@ -558,6 +534,9 @@ void MatrixPetsc<T>::updatePCFieldSplit( PC & pc )
 #endif
             CHKERRABORT( this->comm(),ierr );
         }
+
+
+
     }
 
 #endif
@@ -677,7 +656,7 @@ void MatrixPetsc<T>::close () const
     //     return;
 
     int ierr=0;
-
+    CHECK( M_mat ) << "invalid matrix";
     ierr = MatAssemblyBegin ( M_mat, MAT_FINAL_ASSEMBLY );
     CHKERRABORT( this->comm(),ierr );
     ierr = MatAssemblyEnd   ( M_mat, MAT_FINAL_ASSEMBLY );
@@ -1177,12 +1156,16 @@ MatrixPetsc<T>::zeroRows( std::vector<int> const& rows, Vector<value_type> const
     MatSetOption( M_mat,MAT_KEEP_ZEROED_ROWS );
 #endif
 
-    if ( on_context.test( ON_ELIMINATION_SYMMETRIC ) )
+    if ( on_context.test( OnContext::ELIMINATION_SYMMETRIC ) )
     {
         //PetscErrorCode  MatZeroRowsColumns(Mat mat,PetscInt numRows,const PetscInt rows[],PetscScalar diag,Vec x,Vec b)
         VectorPetsc<T>* prhs = dynamic_cast<VectorPetsc<T>*> ( &rhs );
         const VectorPetsc<T>* pvalues = dynamic_cast<const VectorPetsc<T>*> ( &values );
+#if (PETSC_VERSION_MAJOR >= 3) && (PETSC_VERSION_MINOR >= 2)
         MatZeroRowsColumns(M_mat, rows.size(), rows.data(), 1.0, pvalues->vec(), prhs->vec() );
+#else
+        MatZeroRows( M_mat, rows.size(), rows.data(), 1.0 );
+#endif
         PetscBool b;
         MatIsSymmetric( M_mat, 1e-13, &b );
         LOG(INFO) << "Mat is symmetric : " << b;
@@ -1194,7 +1177,7 @@ MatrixPetsc<T>::zeroRows( std::vector<int> const& rows, Vector<value_type> const
     ierr = MatGetOwnershipRange( M_mat, &start, &stop );
     CHKERRABORT( this->comm(),ierr );
 
-    if ( on_context.test( ON_ELIMINATION_KEEP_DIAGONAL ) )
+    if ( on_context.test( OnContext::ELIMINATION_KEEP_DIAGONAL ) )
     {
         VectorPetsc<value_type> diag( this->size1(), stop-start );
         MatGetDiagonal( M_mat, diag.vec() );
@@ -1312,10 +1295,22 @@ MatrixPetsc<T>::transpose( MatrixSparse<value_type>& Mt ) const
 {
     this->close();
 
-    MatrixPetsc<T>* Atrans = dynamic_cast<MatrixPetsc<T>*> ( &Mt );
+    MatrixPetsc<T>* Atrans;
+    if ( this->comm().size()>1 )
+    {
+        Atrans = dynamic_cast<MatrixPetscMPI<T>*> ( &Mt );
+    }
+    else
+    {
+        Atrans = dynamic_cast<MatrixPetsc<T>*> ( &Mt );
+    }
 
-    int ierr = PETSc::MatDestroy( Atrans->M_mat );
-    CHKERRABORT( this->comm(),ierr );
+    int ierr = 0;
+    if ( Atrans->isInitialized() )
+    {
+        ierr = PETSc::MatDestroy( Atrans->M_mat );
+        CHKERRABORT( this->comm(),ierr );
+    }
 
 #if (PETSC_VERSION_MAJOR >= 3)
     ierr = MatTranspose( M_mat, MAT_INITIAL_MATRIX,&Atrans->M_mat );
@@ -1346,6 +1341,7 @@ MatrixPetsc<T>::transpose( MatrixSparse<value_type>& Mt ) const
     }
 
     Mt.setGraph( this->graph()->transpose() );
+    Mt.setInitialized( true );
 
 }
 
@@ -1631,9 +1627,9 @@ void MatrixPetsc<T>::zeroEntriesDiagonal()
 
 template <typename T>
 inline
-MatrixPetscMPI<T>::MatrixPetscMPI()
+MatrixPetscMPI<T>::MatrixPetscMPI( WorldComm const& worldComm )
     :
-    super()
+    super( worldComm )
 {}
 
 //----------------------------------------------------------------------------------------------------//
@@ -1675,7 +1671,7 @@ void MatrixPetscMPI<T>::init( const size_type m,
                               graph_ptrtype const& graph )
 {
     //this->comm().globalComm().barrier();
-    //std::cout << "MatrixPetscMPI<T>::init with graph start on proc"<< this->comm().globalRank() << "("<<this->comm().godRank() <<")" << std::endl;
+    VLOG(1) << "MatrixPetscMPI<T>::init with graph start on proc"<< this->comm().globalRank() << "("<<this->comm().godRank() <<")" << std::endl;
 
     this->setGraph( graph );
     this->graph()->close();
@@ -2520,7 +2516,7 @@ MatrixPetscMPI<T>::zeroRows( std::vector<int> const& rows,
     else
         {
 #if (PETSC_VERSION_MAJOR >= 3) && (PETSC_VERSION_MINOR >= 2)
-            if ( on_context.test( ON_ELIMINATION_SYMMETRIC ) )
+            if ( on_context.test( OnContext::ELIMINATION_SYMMETRIC ) )
             {
                 MatZeroRowsColumnsLocal(this->M_mat, rows.size(), rows.data(), 1.0, pvalues->vec(), prhs->vec() );
             }
@@ -2620,6 +2616,8 @@ MatrixPetscMPI<T>::energy( Vector<value_type> const& __v,
                            Vector<value_type> const& __u,
                            bool transpose ) const
 {
+    int ierr = 0;
+
     this->close();
 
     PetscScalar e;
@@ -2631,16 +2629,18 @@ MatrixPetscMPI<T>::energy( Vector<value_type> const& __v,
         VectorPetscMPI<value_type> z( this->mapRowPtr() );
 
         if ( !transpose )
-            MatMult( this->mat(), u.vec(), z.vec() );
+            ierr = MatMult( this->mat(), u.vec(), z.vec() );
         else
-            MatMultTranspose( this->mat(), u.vec(), z.vec() );
+            ierr = MatMultTranspose( this->mat(), u.vec(), z.vec() );
+        CHKERRABORT( this->comm(),ierr );
 
-        VecDot( v.vec(), z.vec(), &e );
+        ierr = VecDot( v.vec(), z.vec(), &e );
+        CHKERRABORT( this->comm(),ierr );
     }
 
     else
     {
-        VectorPetscMPI<value_type> u( this->mapRowPtr() );
+        VectorPetscMPI<value_type> u( this->mapColPtr() );
         {
             //size_type s = u.localSize();
             size_type s = u.map().nLocalDofWithGhost();
@@ -2665,12 +2665,13 @@ MatrixPetscMPI<T>::energy( Vector<value_type> const& __v,
         v.close();
 
         if ( !transpose )
-            MatMult( this->mat(), u.vec() , z.vec() );
-
+            ierr = MatMult( this->mat(), u.vec() , z.vec() );
         else
-            MatMultTranspose( this->mat(), u.vec(), z.vec() );
+            ierr = MatMultTranspose( this->mat(), u.vec(), z.vec() );
+        CHKERRABORT( this->comm(),ierr );
 
-        VecDot( v.vec(), z.vec(), &e );
+        ierr = VecDot( v.vec(), z.vec(), &e );
+        CHKERRABORT( this->comm(),ierr );
     }
 
     return e;
