@@ -103,6 +103,9 @@ public:
     typedef DataMap datamap_type;
     typedef boost::shared_ptr<datamap_type> datamap_ptrtype;
 
+    typedef typename datamap_type::indexsplit_type indexsplit_type;
+    typedef typename datamap_type::indexsplit_ptrtype indexsplit_ptrtype;
+
     /**
      * Constructor; initializes the matrix to be empty, without any
      * structure, i.e.  the matrix is not usable at all. This
@@ -115,7 +118,7 @@ public:
      * the matrix before usage with
      * \p init(...).
      */
-    MatrixSparse ();
+    MatrixSparse( WorldComm const& worldComm=Environment::worldComm() );
 
     MatrixSparse( datamap_ptrtype const& dmRow, datamap_ptrtype const& dmCol, WorldComm const& worldComm=Environment::worldComm() );
 
@@ -205,47 +208,20 @@ public:
                         const size_type n_l,
                         graph_ptrtype const& graph ) = 0;
 
-#if 0
     /**
-     *
+     * set the indexSplit associated to the sparse matrix
      */
-    template<typename DomainSpace, typename ImageSpace>
-    void initIndexSplit( DomainSpace const& dm, ImageSpace const& im )
+    virtual void setIndexSplit( indexsplit_ptrtype const& is )
     {
-        auto nSpace = DomainSpace::element_type::nSpaces;
-
-        if ( nSpace>1 )
-        {
-            //std::cout << "\n Debug : nSpace " << nSpace << "\n";
-            std::vector < std::vector<int> > is( nSpace );
-            uint cptSpaces=0;
-            uint start=0;
-            auto result = boost::make_tuple( cptSpaces,start );
-
-            std::vector < std::vector<int> > indexSplit( nSpace );
-            //detail::computeNDofForEachSpace cndof(nSpace);
-            //detail::computeNDofForEachSpace cndof(indexSplit);
-            boost::fusion::fold( dm->functionSpaces(), result,  cndof );
-
-            this->setIndexSplit( indexSplit );
-        }
-
-    }
-#endif
-    /**
-     *
-     */
-    virtual void setIndexSplit( std::vector< std::vector<size_type> > const &_indexSplit )
-    {
-        M_IndexSplit=_indexSplit;
+        M_indexSplit = is;
     }
 
     /**
-     *
+     * \return the indexSplit associated to the sparse matrix
      */
-    std::vector< std::vector<size_type> > indexSplit() const
+    indexsplit_ptrtype const& indexSplit() const
     {
-        return M_IndexSplit;
+        return M_indexSplit;
     }
 
     /**
@@ -768,7 +744,7 @@ protected:
 
     Context M_mprop;
 
-    std::vector < std::vector<size_type> > M_IndexSplit;
+    indexsplit_ptrtype M_indexSplit;
 
     /**
      * data distribution map of the vector over the processors
@@ -788,7 +764,8 @@ typedef boost::shared_ptr<d_sparse_matrix_type> sparse_matrix_ptrtype;
 // MatrixSparse inline members
 template <typename T>
 inline
-MatrixSparse<T>::MatrixSparse () :
+MatrixSparse<T>::MatrixSparse( WorldComm const& worldComm ) :
+    M_worldComm( worldComm ),
     M_is_initialized( false ),
     M_mprop( NON_HERMITIAN )
 {}
@@ -886,6 +863,18 @@ std::ostream& operator << ( std::ostream& os, const MatrixSparse<T>& m )
     return os;
 }
 
+namespace detail
+{
+template <class MatrixType>
+struct is_matrix_ptr : mpl::false_ {};
+
+template <class MatrixType>
+struct is_matrix_ptr<boost::shared_ptr<MatrixType> >
+        :
+        boost::is_base_of<MatrixSparse<typename MatrixType::value_type>,
+        MatrixType>
+{};
+}
 } // Feel
 
 #endif // #ifndef __sparse_matrix_h__
