@@ -77,7 +77,6 @@ extern "C"
         return 0;
     }
 
-
     PetscErrorCode __feel_petsc_preconditioner_apply( void *ctx, Vec x, Vec y )
     {
         Preconditioner<double> * preconditioner = static_cast<Preconditioner<double>*>( ctx );
@@ -89,6 +88,13 @@ extern "C"
 
         return 0;
     }
+    PetscErrorCode __feel_petsc_preconditioner_view( void *ctx, PetscViewer viewer)
+    {
+        Preconditioner<double> * preconditioner = static_cast<Preconditioner<double>*>( ctx );
+        preconditioner->view();
+        return 0;
+    }
+
 #else
     PetscErrorCode __feel_petsc_preconditioner_setup ( PC pc )
     {
@@ -114,6 +120,15 @@ extern "C"
 
         return 0;
     }
+    PetscErrorCode __feel_petsc_preconditioner_view( PC pc, PetscViewer viewer)
+    {
+        void *ctx;
+        PetscErrorCode ierr = PCShellGetContext( pc,&ctx );
+        CHKERRQ( ierr );
+        Preconditioner<double> * preconditioner = static_cast<Preconditioner<double>*>( ctx );
+        preconditioner->view();
+        return 0;
+    }
 #endif
 } // end extern "C"
 
@@ -215,10 +230,7 @@ void SolverLinearPetsc<T>::init ()
 
         // Set user-specified  solver and preconditioner types
         this->setPetscSolverType();
-        this->setPetscPreconditionerType();
         this->setPetscConstantNullSpace();
-        // sets the software that is used to perform the factorization
-        PetscPCFactorSetMatSolverPackage( M_pc,this->matSolverPackageType() );
 
 
         // Set the options from user-input
@@ -251,9 +263,9 @@ void SolverLinearPetsc<T>::init ()
         ierr = KSPGetType ( M_ksp, &ksp_type );
         CHKERRABORT( this->worldComm().globalComm(),ierr );
 
-        if ( strcmp( ksp_type, "preonly" ) )
+        if ( std::string((char*)ksp_type) == std::string( ( char* )KSPPREONLY ) )
         {
-            ierr = KSPSetInitialGuessNonzero ( M_ksp, PETSC_TRUE );
+            ierr = KSPSetInitialGuessNonzero ( M_ksp, PETSC_FALSE );
             CHKERRABORT( this->worldComm().globalComm(),ierr );
         }
 
@@ -284,6 +296,7 @@ void SolverLinearPetsc<T>::init ()
             PCShellSetContext( M_pc,( void* )this->M_preconditioner.get() );
             PCShellSetSetUp( M_pc,__feel_petsc_preconditioner_setup );
             PCShellSetApply( M_pc,__feel_petsc_preconditioner_apply );
+            PCShellSetView( M_pc,__feel_petsc_preconditioner_view );
 #if PETSC_VERSION_LESS_THAN(3,4,0)
             const PCType pc_type;
 #else
@@ -294,13 +307,18 @@ void SolverLinearPetsc<T>::init ()
 
             VLOG(2) << "preconditioner set as "  << pc_type << "\n";
         }
+        else
+        {
+            this->setPetscPreconditionerType();
+            // sets the software that is used to perform the factorization
+            PetscPCFactorSetMatSolverPackage( M_pc,this->matSolverPackageType() );
+        }
 
         if ( Environment::vm(_name="ksp-monitor",_prefix=this->prefix()).template as<bool>() )
         {
             KSPMonitorSet( M_ksp,KSPMonitorDefault,PETSC_NULL,PETSC_NULL );
         }
 
-
     }
 }
 
@@ -312,8 +330,7 @@ void SolverLinearPetsc<T>::init ()
 
 
 template <typename T>
-//std::pair<unsigned int, typename SolverLinearPetsc<T>::real_type>
-boost::tuple<bool,unsigned int, typename SolverLinearPetsc<T>::real_type>
+typename SolverLinearPetsc<T>::solve_return_type
 SolverLinearPetsc<T>::solve ( MatrixSparse<T> const&  matrix_in,
                               MatrixSparse<T> const&  precond_in,
                               Vector<T> & solution_in,
@@ -381,7 +398,11 @@ SolverLinearPetsc<T>::solve ( MatrixSparse<T> const&  matrix_in,
     // instead of || B*b ||. In the case of right preconditioner or if
     // KSPSetNormType(ksp,KSP_NORM_UNPRECONDIITONED) is used there is no B in
     // the above formula. UIRNorm is short for Use Initial Residual Norm.
+#if PETSC_VERSION_GREATER_OR_EQUAL_THAN(3,4,4)
+    KSPConvergedDefaultSetUIRNorm( M_ksp );
+#else
     KSPDefaultConvergedSetUIRNorm( M_ksp );
+#endif
 
 
     // Solve the linear system
@@ -429,7 +450,11 @@ SolverLinearPetsc<T>::solve ( MatrixSparse<T> const&  matrix_in,
     // instead of || B*b ||. In the case of right preconditioner or if
     // KSPSetNormType(ksp,KSP_NORM_UNPRECONDIITONED) is used there is no B in
     // the above formula. UIRNorm is short for Use Initial Residual Norm.
+#if PETSC_VERSION_GREATER_OR_EQUAL_THAN(3,4,4)
+    KSPConvergedDefaultSetUIRNorm( M_ksp );
+#else
     KSPDefaultConvergedSetUIRNorm( M_ksp );
+#endif
 
     // Solve the linear system
     if ( transpose )
@@ -497,6 +522,9 @@ SolverLinearPetsc<T>::solve ( MatrixSparse<T> const&  matrix_in,
     KSPConvergedReason reason;
     KSPGetConvergedReason( M_ksp,&reason );
 
+    if ( option( _prefix=this->prefix(), _name="ksp-view" ).template as<bool>() )
+        check( KSPView( M_ksp, PETSC_VIEWER_STDOUT_WORLD ) );
+
     if ( reason==KSP_DIVERGED_INDEFINITE_PC )
     {
         LOG(INFO) << "[solverlinearpetsc] Divergence because of indefinite preconditioner;\n";
@@ -528,13 +556,13 @@ SolverLinearPetsc<T>::solve ( MatrixSparse<T> const&  matrix_in,
 #endif
     // return the # of its. and the final residual norm.
     //return std::make_pair(its, final_resid);
-    return boost::make_tuple( hasConverged, its, final_resid );
+    return solve_return_type( boost::make_tuple( hasConverged, its, final_resid ) );
 
 
 }
 
 template <typename T>
-boost::tuple<bool,unsigned int, typename SolverLinearPetsc<T>::real_type>
+typename SolverLinearPetsc<T>::solve_return_type
 SolverLinearPetsc<T>::solve ( MatrixShell<T>  const &mat,
                               Vector<T> & x,
                               Vector<T> const& b,
@@ -543,7 +571,7 @@ SolverLinearPetsc<T>::solve ( MatrixShell<T>  const &mat,
                               bool transpose )
 {
     LOG(ERROR) << "invalid call to solve() using matshell\n";
-    return boost::make_tuple( false, 0, 0 );
+    return solve_return_type( boost::make_tuple( false, 0, 0 ) );
 }
 
 template <typename T>
@@ -684,6 +712,11 @@ SolverLinearPetsc<T>::setPetscSolverType()
         CHKERRABORT( this->worldComm().globalComm(),ierr );
         return;
 
+    case FGMRES:
+        ierr = KSPSetType ( M_ksp, ( char* ) KSPFGMRES );
+        CHKERRABORT( this->worldComm().globalComm(),ierr );
+        return;
+
     case RICHARDSON:
         ierr = KSPSetType ( M_ksp, ( char* ) KSPRICHARDSON );
         CHKERRABORT( this->worldComm().globalComm(),ierr );
@@ -698,6 +731,11 @@ SolverLinearPetsc<T>::setPetscSolverType()
         CHKERRABORT( this->worldComm().globalComm(),ierr );
         return;
 
+    case PREONLY :
+        ierr = KSPSetType ( M_ksp, ( char* ) KSPPREONLY );
+        CHKERRABORT( this->worldComm().globalComm(),ierr );
+        return;
+
     default:
         std::cerr << "ERROR:  Unsupported PETSC Solver: "
                   << this->solverType()               << std::endl
