@@ -125,14 +125,6 @@ extern "C"
         Feel::SolverNonLinearPetsc<double>* solver =
             static_cast<Feel::SolverNonLinearPetsc<double>*> ( ctx );
 
-#if !defined(FEELPP_ENABLE_MPI_MODE)
-        boost::shared_ptr<Feel::Vector<double> > R( new Feel::VectorPetsc<double>( r ) );
-        boost::shared_ptr<Feel::MatrixSparse<double> >  PC;
-        boost::shared_ptr<Feel::Vector<double> > X_global( new Feel::VectorPetsc<double>( x ) );
-        boost::shared_ptr<Feel::Vector<double> > X_local( new Feel::VectorPetsc<double>( X_global->size() ) );
-
-        X_global->localize ( *X_local );
-#else
         boost::shared_ptr<Feel::Vector<double> > R;
         boost::shared_ptr<Feel::Vector<double> > X_global;
 
@@ -148,8 +140,6 @@ extern "C"
             X_global.reset( new Feel::VectorPetsc<double>( x ) );
         }
 
-#endif
-
         //if (solver->residual != NULL) solver->residual (X_local, R);
         if ( solver->residual != NULL ) solver->residual ( X_global, R );
 
@@ -173,15 +163,6 @@ extern "C"
         Feel::SolverNonLinearPetsc<double>* solver =
             static_cast<Feel::SolverNonLinearPetsc<double>*> ( ctx );
 
-#if !defined(FEELPP_ENABLE_MPI_MODE)
-        boost::shared_ptr<Feel::Vector<double> > R;
-        boost::shared_ptr<Feel::MatrixSparse<double> >  PC( new Feel::MatrixPetsc<double>( *pc ) );
-        boost::shared_ptr<Feel::MatrixSparse<double> >  Jac( new Feel::MatrixPetsc<double>( *jac ) );
-        boost::shared_ptr<Feel::Vector<double> > X_global( new Feel::VectorPetsc<double>( x ) );
-        boost::shared_ptr<Feel::Vector<double> > X_local( new Feel::VectorPetsc<double>( X_global->size() ) );
-
-        X_global->localize ( *X_local );
-#else // MPI
         boost::shared_ptr<Feel::MatrixSparse<double> > Jac;
         boost::shared_ptr<Feel::Vector<double> > X_global;
 
@@ -197,8 +178,6 @@ extern "C"
             X_global.reset( new Feel::VectorPetsc<double>( x,solver->mapColPtr() ) );
         }
 
-#endif
-
         //if (solver->jacobian != NULL) solver->jacobian (X_local, PC );
         if ( solver->jacobian != NULL ) solver->jacobian ( X_global, Jac );
 
@@ -465,12 +444,12 @@ void SolverNonLinearPetsc<T>::setReuse ( int jac, int prec )
 template <typename T>
 void SolverNonLinearPetsc<T>::init ()
 {
+    int ierr=0;
     // Initialize the data structures if not done so already.
     if ( !this->initialized() )
     {
         this->M_is_initialized = true;
 
-        int ierr=0;
 
 # if ((PETSC_VERSION_MAJOR == 2) && (PETSC_VERSION_MINOR <= 1) && (PETSC_VERSION_SUBMINOR <= 1))
 
@@ -500,10 +479,12 @@ void SolverNonLinearPetsc<T>::init ()
         ierr = SNESSetFromOptions( M_snes );
         CHKERRABORT( this->worldComm().globalComm(),ierr );
 
-        //int ierr=0;
+#if 1
         // if the non linear solver type is define by the user in the code
         switch ( this->getType() )
         {
+            //LS, TR Newton-type with line search and trust region
+        default:
         case LINE_SEARCH :
         {
 #if PETSC_VERSION_LESS_THAN(3,4,0)
@@ -525,54 +506,57 @@ void SolverNonLinearPetsc<T>::init ()
             CHKERRABORT( this->worldComm().globalComm(),ierr );
         }
         break;
+#if PETSC_VERSION_GREATER_OR_EQUAL_THAN(3,4,0)
+
+        //NRichardson Nonlinear Richardson, usually preconditioned
+        case NRICHARDSON: check( SNESSetType( M_snes, SNESNRICHARDSON ) ); break;
+
+        case NKSPONLY: check( SNESSetType( M_snes, SNESKSPONLY ) ); break;
+            //VIRS, VISS reduced space and semi-smooth method for
+            //variational inequalities
+        case VINEWTONRSLS: check( SNESSetType( M_snes, SNESVINEWTONRSLS ) ); break;
+        case VINEWTONRSTR: check( SNESSetType( M_snes, SNESVINEWTONSSLS ) ); break;
+            //NGMRES Nonlinear GMRES
+        case NGMRES: check( SNESSetType( M_snes, SNESNGMRES ) ); break;
+            //QN Quasi-Newton methods like BFGS
+        case QN: check( SNESSetType( M_snes, SNESQN ) ); break;
+            //Shell Your method, often used as a (nonlinear) preconditioner
+        case NSHELL: check( SNESSetType( M_snes, SNESSHELL ) ); break;
+            //GS Nonlinear Gauss-Seidel sweeps
+        case GS: check( SNESSetType( M_snes, SNESGS ) ); break;
+            //NCG Nonlinear Conjugate Gradients
+        case NCG: check( SNESSetType( M_snes, SNESNCG ) ); break;
+            //FAS Full approximation scheme (nonlinear multigrid)
+        case FAS: check( SNESSetType( M_snes, SNESFAS ) ); break;
+            //MS Multi-stage smoothers (in FAS for hyperbolic problems)
+        case MS: check( SNESSetType( M_snes, SNESMS ) ); break;
+        case NASM: check( SNESSetType( M_snes, SNESNASM ) ); break;
+        case ANDERSON: check( SNESSetType( M_snes, SNESANDERSON ) ); break;
+        case ASPIN: check( SNESSetType( M_snes, SNESASPIN ) ); break;
+#endif
+
 
         case SELECT_IN_ARGLIST:
             // no-op
             break;
         }
-
-
-        double __relResTol,__absResTol,__absSolTol;
-        int __nbItMax, __nbEvalFuncMax;
-
-        if ( this->getAbsoluteResidualTol()==0 )
-        {
-            ierr = SNESGetTolerances( M_snes, &__absResTol, PETSC_NULL, PETSC_NULL, PETSC_NULL, PETSC_NULL );
-            CHKERRABORT( this->worldComm().globalComm(),ierr );
-        }
-
-        else __absResTol = this->getAbsoluteResidualTol();
-
-        if ( this->getRelativeResidualTol()==0 )
-        {
-            ierr = SNESGetTolerances( M_snes, PETSC_NULL, &__relResTol, PETSC_NULL, PETSC_NULL, PETSC_NULL );
-            CHKERRABORT( this->worldComm().globalComm(),ierr );
-        }
-
-        else __relResTol = this->getRelativeResidualTol();
-
-        if ( this->getAbsoluteSolutionTol()==0 )
-        {
-            ierr = SNESGetTolerances( M_snes, PETSC_NULL, PETSC_NULL, &__absSolTol, PETSC_NULL, PETSC_NULL );
-            CHKERRABORT( this->worldComm().globalComm(),ierr );
-        }
-
-        else __absSolTol = this->getAbsoluteSolutionTol();
-
-        if ( this->getNbItMax()==0 )
-        {
-            ierr = SNESGetTolerances( M_snes, PETSC_NULL, PETSC_NULL, PETSC_NULL, &__nbItMax, PETSC_NULL );
-            CHKERRABORT( this->worldComm().globalComm(),ierr );
-        }
-
-        else __nbItMax = this->getNbItMax();
-
-        ierr = SNESGetTolerances( M_snes, PETSC_NULL, PETSC_NULL, PETSC_NULL, PETSC_NULL, &__nbEvalFuncMax );
-        CHKERRABORT( this->worldComm().globalComm(),ierr );
-
-        ierr = SNESSetTolerances( M_snes,__absResTol,__relResTol,__absSolTol,__nbItMax,__nbEvalFuncMax );
-        CHKERRABORT( this->worldComm().globalComm(),ierr );
-
+#else
+        std::string s = option(_name="snes-type",_prefix=this->prefix()).template as<std::string>();
+        DVLOG(1) << "snes type: " << s;
+        check( SNESSetType( M_snes, s.c_str() ) );
+#if 0
+        //check( SNESNGMRESSetRestartType( M_snes, 5 ) );
+        SNES pc;
+        check( SNESGetPC( M_snes, &pc ) );
+        SNESType spc;
+        check( SNESGetType( pc, &spc ) );
+        LOG(INFO) << "npc snes type: " << spc;
+        //check( SNESSetType( pc, SNESNRICHARDSON ) );
+        check( SNESSetType( pc, SNESNEWTONLS ) );
+        //check( SNESSetIterationNumber(pc, 5 ) );
+        check( SNESMonitorSet( pc,SNESMonitorDefault,PETSC_NULL,PETSC_NULL ) );
+#endif
+#endif
         //KSP ksp;
         ierr = SNESGetKSP ( M_snes, &M_ksp );
         CHKERRABORT( this->worldComm().globalComm(),ierr );
@@ -582,13 +566,13 @@ void SolverNonLinearPetsc<T>::init ()
 
         // Set user-specified  solver and preconditioner types
         this->setPetscKspSolverType();
-        this->setPetscPreconditionerType();
+        //this->setPetscPreconditionerType();
         //this->setPetscConstantNullSpace();
         // sets the software that is used to perform the factorization
-        PetscPCFactorSetMatSolverPackage( M_pc,this->matSolverPackageType() );
+        //PetscPCFactorSetMatSolverPackage( M_pc,this->matSolverPackageType() );
 
 
-        if ( this->M_preconditioner /*&& this->preconditionerType()==PreconditionerType::SHELL_PRECOND*/ )
+        if ( this->M_preconditioner )
         {
             PCSetType( M_pc, PCSHELL );
             PCShellSetContext( M_pc,( void* )this->M_preconditioner.get() );
@@ -596,6 +580,13 @@ void SolverNonLinearPetsc<T>::init ()
             //Re-Use the shell functions from petsc_linear_solver
             PCShellSetSetUp( M_pc,__feel_petsc_preconditioner_setup );
             PCShellSetApply( M_pc,__feel_petsc_preconditioner_apply );
+            PCShellSetView( M_pc,__feel_petsc_preconditioner_view );
+        }
+        else
+        {
+            this->setPetscPreconditionerType();
+            // sets the software that is used to perform the factorization
+            PetscPCFactorSetMatSolverPackage( M_pc,this->matSolverPackageType() );
         }
 
         if ( Environment::vm(_name="snes-monitor",_prefix=this->prefix()).template as<bool>() )
@@ -610,15 +601,58 @@ void SolverNonLinearPetsc<T>::init ()
             CHKERRABORT( this->worldComm().globalComm(),ierr );
         }
 
+    } // if ( !this->initialized() )
+
+
+
+    double __relResTol,__absResTol,__absSolTol;
+    int __nbItMax, __nbEvalFuncMax;
+
+    if ( this->getAbsoluteResidualTol()==0 )
+    {
+        ierr = SNESGetTolerances( M_snes, &__absResTol, PETSC_NULL, PETSC_NULL, PETSC_NULL, PETSC_NULL );
+        CHKERRABORT( this->worldComm().globalComm(),ierr );
     }
 
+    else __absResTol = this->getAbsoluteResidualTol();
+
+    if ( this->getRelativeResidualTol()==0 )
+    {
+        ierr = SNESGetTolerances( M_snes, PETSC_NULL, &__relResTol, PETSC_NULL, PETSC_NULL, PETSC_NULL );
+        CHKERRABORT( this->worldComm().globalComm(),ierr );
+    }
+
+    else __relResTol = this->getRelativeResidualTol();
+
+    if ( this->getAbsoluteSolutionTol()==0 )
+    {
+        ierr = SNESGetTolerances( M_snes, PETSC_NULL, PETSC_NULL, &__absSolTol, PETSC_NULL, PETSC_NULL );
+        CHKERRABORT( this->worldComm().globalComm(),ierr );
+    }
+
+    else __absSolTol = this->getAbsoluteSolutionTol();
+
+    if ( this->getNbItMax()==0 )
+    {
+        ierr = SNESGetTolerances( M_snes, PETSC_NULL, PETSC_NULL, PETSC_NULL, &__nbItMax, PETSC_NULL );
+        CHKERRABORT( this->worldComm().globalComm(),ierr );
+    }
+
+    else __nbItMax = this->getNbItMax();
+
+    ierr = SNESGetTolerances( M_snes, PETSC_NULL, PETSC_NULL, PETSC_NULL, PETSC_NULL, &__nbEvalFuncMax );
+    CHKERRABORT( this->worldComm().globalComm(),ierr );
+
+    ierr = SNESSetTolerances( M_snes,__absResTol,__relResTol,__absSolTol,__nbItMax,__nbEvalFuncMax );
+    CHKERRABORT( this->worldComm().globalComm(),ierr );
+
 
 }
 
 
 
 template <typename T>
-std::pair< int, typename SolverNonLinearPetsc<T>::real_type>
+typename SolverNonLinearPetsc<T>::solve_return_type
 SolverNonLinearPetsc<T>::solve ( sparse_matrix_ptrtype&  jac_in,  // System Jacobian Matrix
                                  vector_ptrtype& x_in,    // Solution vector
                                  vector_ptrtype& r_in,    // Residual vector
@@ -633,11 +667,6 @@ SolverNonLinearPetsc<T>::solve ( sparse_matrix_ptrtype&  jac_in,  // System Jaco
 
     int ierr=0;
 
-#if !defined(FEELPP_ENABLE_MPI_MODE)
-    MatrixPetsc<T>* jac = dynamic_cast<MatrixPetsc<T>*>( jac_in.get() );
-    VectorPetsc<T>* x   = dynamic_cast<VectorPetsc<T>*>( x_in.get() );
-    VectorPetsc<T>* r   = dynamic_cast<VectorPetsc<T>*>( r_in.get() );
-#else // MPI
     MatrixPetsc<T>* jac;
     VectorPetsc<T>* x;
     VectorPetsc<T>* r;
@@ -659,9 +688,6 @@ SolverNonLinearPetsc<T>::solve ( sparse_matrix_ptrtype&  jac_in,  // System Jaco
     this->setMapRow( jac_in->mapRowPtr() );
     this->setMapCol( jac_in->mapColPtr() );
 
-#endif
-
-
     // We cast to pointers so we can be sure that they succeeded
     // by comparing the result against NULL.
     assert( jac != NULL );
@@ -672,6 +698,7 @@ SolverNonLinearPetsc<T>::solve ( sparse_matrix_ptrtype&  jac_in,  // System Jaco
     assert( r->vec()   != NULL );
 
     int n_iterations =0;
+    PetscReal valfnorm = 0;
 
     ierr = SNESSetFunction ( M_snes, r->vec(), __feel_petsc_snes_residual, this );
     CHKERRABORT( this->worldComm().globalComm(),ierr );
@@ -741,7 +768,10 @@ SolverNonLinearPetsc<T>::solve ( sparse_matrix_ptrtype&  jac_in,  // System Jaco
     CHKERRABORT( this->worldComm().globalComm(),ierr );
     LOG(INFO) << "[SolverNonLinearPetsc] number of nonlinear iterations = " << n_iterations << "\n";
 
-    for ( int i=0; i<n_iterations+1; i++ )
+    ierr = SNESGetFunctionNorm( M_snes,&valfnorm );
+    CHKERRABORT( this->worldComm().globalComm(),ierr );
+
+    for ( int i=0; i<50/*n_iterations+1*/; i++ )
     {
         LOG(INFO) << "iteration " << i << ": Linear iterations : " << hist_its[i] << " Function norm = " << history[i] << "\n";
     }
@@ -750,7 +780,10 @@ SolverNonLinearPetsc<T>::solve ( sparse_matrix_ptrtype&  jac_in,  // System Jaco
     SNESGetConvergedReason( M_snes,&reason );
     LOG(INFO) << "[solvernonlinearpetsc] convergence reason : " << reason << "\n";
 
-    if ( reason<0 )
+    if ( option( _prefix=this->prefix(), _name="snes-view" ).template as<bool>() )
+        check( SNESView( M_snes, PETSC_VIEWER_STDOUT_WORLD ) );
+    bool hasConverged = reason>0;
+    if ( !hasConverged )
     {
         LOG(ERROR) << "Nonlinear solve did not converge due to " << PetscConvertSNESReasonToString(reason)
                    << " iterations " << n_iterations << std::endl;
@@ -765,13 +798,10 @@ SolverNonLinearPetsc<T>::solve ( sparse_matrix_ptrtype&  jac_in,  // System Jaco
                       << " iterations " << n_iterations << std::endl;
     }
 
-
-#if 0
-    this->clear();
-#endif
     // return the # of its. and the final residual norm.  Note that
     // n_iterations may be zero for PETSc versions 2.2.x and greater.
-    return std::make_pair( reason, 0. );
+    //return std::make_pair( reason, 0. );
+    return solve_return_type( boost::make_tuple( hasConverged, n_iterations, valfnorm/*history[std::min(n_iterations,49)]*/ ) );
 }
 
 
@@ -879,10 +909,14 @@ SolverNonLinearPetsc<T>::solve ( dense_matrix_type&  jac_in,  // System Jacobian
     SNESConvergedReason reason;
     SNESGetConvergedReason( M_snes,&reason );
 
+
+    if ( option( _prefix=this->prefix(), _name="snes-view" ).template as<bool>() )
+        check( SNESView( M_snes,PETSC_VIEWER_STDOUT_SELF ) );
+
     //LOG(INFO) << "[solvernonlinearpetsc] convergence reason : " << reason << "\n";
     if ( reason<0 )
     {
-        DVLOG(2)  << "[solvernonlinearpetsc] not converged (see petscsnes.h for an explanation): " << reason << "\n";
+        VLOG(1)  << "[solvernonlinearpetsc] not converged (see petscsnes.h for an explanation): " << reason << "\n";
     }
 
     this->clear();
@@ -987,10 +1021,13 @@ SolverNonLinearPetsc<T>::solve ( map_dense_matrix_type&  jac_in,  // System Jaco
     SNESConvergedReason reason;
     SNESGetConvergedReason( M_snes,&reason );
 
+    if ( option( _prefix=this->prefix(), _name="snes-view" ).template as<bool>() )
+        check( SNESView( M_snes,PETSC_VIEWER_STDOUT_SELF ) );
+
     //LOG(INFO) << "[solvernonlinearpetsc] convergence reason : " << reason << "\n";
     if ( reason<0 )
     {
-        DVLOG(2)  << "[solvernonlinearpetsc] not converged (see petscsnes.h for an explanation): " << reason << "\n";
+        VLOG(1)  << "[solvernonlinearpetsc] not converged (see petscsnes.h for an explanation): " << reason << "\n";
     }
 
     this->clear();
@@ -1067,6 +1104,11 @@ SolverNonLinearPetsc<T>::setPetscKspSolverType()
         CHKERRABORT( this->comm(),ierr );
         return;
 
+    case FGMRES:
+        ierr = KSPSetType ( M_ksp, ( char* ) KSPFGMRES );
+        CHKERRABORT( this->worldComm().globalComm(),ierr );
+        return;
+
     case RICHARDSON:
         ierr = KSPSetType ( M_ksp, ( char* ) KSPRICHARDSON );
         CHKERRABORT( this->comm(),ierr );
