@@ -97,7 +97,8 @@ Gmsh::Gmsh( int nDim, int nOrder, WorldComm const& worldComm )
     M_recombine( 0 ),
     M_structured( false ),
     M_refine_levels( 0 ),
-    M_substructuring( false )
+    M_substructuring( false ),
+    M_periodic()
 {
     this->setReferenceDomain();
     setX( std::make_pair( 0., 1.) );
@@ -125,7 +126,8 @@ Gmsh::Gmsh( Gmsh const & __g )
     M_recombine( __g.M_recombine ),
     M_structured( __g.M_structured ),
     M_refine_levels( __g.M_refine_levels ),
-    M_substructuring( __g.M_substructuring )
+    M_substructuring( __g.M_substructuring ),
+    M_periodic( __g.M_periodic )
 {}
 Gmsh::~Gmsh()
 {}
@@ -144,6 +146,8 @@ Gmsh::operator=( Gmsh const& __g )
         M_usePhysicalNames = __g.M_usePhysicalNames;
         M_shear = __g.M_shear;
         M_refine_levels = __g.M_refine_levels;
+        M_periodic = __g.M_periodic;
+        M_worldComm = __g.M_worldComm;
     }
 
     return *this;
@@ -159,7 +163,7 @@ Gmsh::New( po::variables_map const& vm )
 }
 
 boost::shared_ptr<Gmsh>
-Gmsh::New( std::string const& shape, uint16_type d, uint16_type o, std::string const& ct )
+Gmsh::New( std::string const& shape, uint16_type d, uint16_type o, std::string const& ct, WorldComm const& wc )
 {
     std::ostringstream ostr;
 
@@ -170,6 +174,7 @@ Gmsh::New( std::string const& shape, uint16_type d, uint16_type o, std::string c
         ostr << shape << "(" << d << "," << o << "," << boost::to_lower_copy( ct ) << ")";
 
     boost::shared_ptr<Gmsh> gmsh_ptr( Gmsh::Factory::type::instance().createObject( ostr.str() ) );
+    gmsh_ptr->setWorldComm( wc );
     return gmsh_ptr;
 }
 
@@ -838,34 +843,7 @@ const bool meshs112ts = Gmsh::Factory::type::instance().registerProduct( "hyperc
 
 
 
-boost::shared_ptr<Mesh<Simplex<1> > >
-unitSegment( double h )
-{
-    return createGMSHMesh(_mesh=new Mesh<Simplex<1> >,
-                          _desc=domain( _name="segment",
-                                        _shape="hypercube",
-                                        _dim=1,
-                                        _h=h ) );
-}
 
-boost::shared_ptr<Mesh<Simplex<2> > >
-unitSquare( double h )
-{
-    return createGMSHMesh(_mesh=new Mesh<Simplex<2> >,
-                          _desc=domain( _name="square",
-                                        _shape="hypercube",
-                                        _dim=2,
-                                        _h=h ) );
-}
 
-boost::shared_ptr<Mesh<Simplex<3> > >
-unitCube( double h )
-{
-    return createGMSHMesh(_mesh=new Mesh<Simplex<3> >,
-                          _desc=domain( _name="cube",
-                                        _shape="hypercube",
-                                        _dim=3,
-                                        _h= h ) );
-}
 
 } // Feel
