@@ -107,45 +107,45 @@ public:
             multi_index::ordered_unique<
                 multi_index::composite_key<element_type,
                                            multi_index::const_mem_fun<element_type,
-                                                                      uint16_type,
+                                                                      rank_type,
                                                                       &element_type::processId>,
                                            multi_index::const_mem_fun<element_type,
                                                                       size_type,
                                                                       &element_type::id> > >,
             // sort by less<int> on marker
-            multi_index::ordered_non_unique<multi_index::tag<detail::by_marker>,
+            multi_index::ordered_non_unique<multi_index::tag<Feel::detail::by_marker>,
                                             multi_index::composite_key<element_type,
                                                                        multi_index::const_mem_fun<element_type,
                                                                                                   Marker1 const&,
                                                                                                   &element_type::marker>,
                                                                        multi_index::const_mem_fun<element_type,
-                                                                                                  uint16_type,
+                                                                                                  rank_type,
                                                                                                   &element_type::processId> > >,
             // sort by less<int> on marker
-            multi_index::ordered_non_unique<multi_index::tag<detail::by_marker2>,
+            multi_index::ordered_non_unique<multi_index::tag<Feel::detail::by_marker2>,
                                             multi_index::composite_key<element_type,
                                                                        multi_index::const_mem_fun<element_type,
                                                                                                   Marker2 const&,
                                                                                                   &element_type::marker2>,
                                                                        multi_index::const_mem_fun<element_type,
-                                                                                                  uint16_type,
+                                                                                                  rank_type,
                                                                                                   &element_type::processId> > >,
 
             // sort by less<int> on marker
-            multi_index::ordered_non_unique<multi_index::tag<detail::by_marker3>,
+            multi_index::ordered_non_unique<multi_index::tag<Feel::detail::by_marker3>,
                                             multi_index::composite_key<element_type,
                                                                        multi_index::const_mem_fun<element_type,
                                                                                                   Marker3 const&,
                                                                                                   &element_type::marker3>,
                                                                        multi_index::const_mem_fun<element_type,
-                                                                                                  uint16_type,
+                                                                                                  rank_type,
                                                                                                   &element_type::processId> > >,
 
             // sort by less<int> on boundary
-            multi_index::ordered_non_unique<multi_index::tag<detail::by_location>,
+            multi_index::ordered_non_unique<multi_index::tag<Feel::detail::by_location>,
                                             multi_index::composite_key<element_type,
                                                                        multi_index::const_mem_fun<element_type,
-                                                                                                  uint16_type,
+                                                                                                  rank_type,
                                                                                                   &element_type::processId>,
                                                                        multi_index::const_mem_fun<element_type,
                                                                                                   bool,
@@ -156,13 +156,13 @@ public:
 
 
             // sort by less<int> on processId
-            multi_index::ordered_non_unique<multi_index::tag<detail::by_pid>,
+            multi_index::ordered_non_unique<multi_index::tag<Feel::detail::by_pid>,
                                             multi_index::const_mem_fun<element_type,
-                                                                       uint16_type,
+                                                                       rank_type,
                                                                        &element_type::processId> >,
 
             // sort by less<int> on processId
-            multi_index::ordered_non_unique<multi_index::tag<detail::by_ghostcell>,
+            multi_index::ordered_non_unique<multi_index::tag<Feel::detail::by_ghostcell>,
                                             multi_index::const_mem_fun<element_type,
                                                                        bool,
                                                                        &element_type::isGhostCell> >
@@ -175,30 +175,30 @@ public:
     typedef typename elements_type::const_iterator element_const_iterator;
 
     // marker
-    typedef typename elements_type::template index<detail::by_marker>::type marker_elements;
+    typedef typename elements_type::template index<Feel::detail::by_marker>::type marker_elements;
     typedef typename marker_elements::iterator marker_element_iterator;
     typedef typename marker_elements::const_iterator marker_element_const_iterator;
 
     // marker2
-    typedef typename elements_type::template index<detail::by_marker2>::type marker2_elements;
+    typedef typename elements_type::template index<Feel::detail::by_marker2>::type marker2_elements;
     typedef typename marker2_elements::iterator marker2_element_iterator;
     typedef typename marker2_elements::const_iterator marker2_element_const_iterator;
 
     // marker3
-    typedef typename elements_type::template index<detail::by_marker3>::type marker3_elements;
+    typedef typename elements_type::template index<Feel::detail::by_marker3>::type marker3_elements;
     typedef typename marker3_elements::iterator marker3_element_iterator;
     typedef typename marker3_elements::const_iterator marker3_element_const_iterator;
 
-    typedef typename elements_type::template index<detail::by_pid>::type pid_elements;
+    typedef typename elements_type::template index<Feel::detail::by_pid>::type pid_elements;
     typedef typename pid_elements::iterator pid_element_iterator;
     typedef typename pid_elements::const_iterator pid_element_const_iterator;
 
 
-    typedef typename elements_type::template index<detail::by_location>::type location_elements;
+    typedef typename elements_type::template index<Feel::detail::by_location>::type location_elements;
     typedef typename location_elements::iterator location_element_iterator;
     typedef typename location_elements::const_iterator location_element_const_iterator;
 
-    typedef typename elements_type::template index<detail::by_ghostcell>::type ghostcell_elements;
+    typedef typename elements_type::template index<Feel::detail::by_ghostcell>::type ghostcell_elements;
     typedef typename ghostcell_elements::iterator ghostcell_element_iterator;
     typedef typename ghostcell_elements::const_iterator ghostcell_element_const_iterator;
 
@@ -208,7 +208,7 @@ public:
     /// \cond disabled
     struct update_element_neighbor_type
     {
-        update_element_neighbor_type( uint16_type n, size_type id )
+        update_element_neighbor_type( rank_type n, size_type id )
             :
             M_pos_neigh( n ),
             M_neigh_id( id )
@@ -220,7 +220,7 @@ public:
         }
 
     private:
-        uint16_type M_pos_neigh;
+        rank_type M_pos_neigh;
         size_type M_neigh_id;
     };
 
@@ -281,7 +281,12 @@ public:
         void operator()( element_type& e )
         {
             for ( int i = 0; i < e.numPoints; ++i )
+            {
                 e.point( i ).addElementGhost( e.processId(),e.id() );
+                // only if point is on interprocess
+                if ( e.point( i ).processId()!=invalid_rank_type_value )
+                    e.point( i ).addNeighborPartitionId( e.processId() );
+            }
         }
     };
 
@@ -295,7 +300,7 @@ public:
         void operator()( element_type& e )
         {
             for ( int i = 0; i < e.numEdges; ++i )
-                detail::updateElementGhostConnectEdgeToElement(e,i,mpl::int_<element_type::nDim>());
+                Feel::detail::updateElementGhostConnectEdgeToElement(e,i,mpl::int_<element_type::nDim>());
         }
     };
 
@@ -321,8 +326,15 @@ public:
     {}
 
     virtual ~Elements()
-    {}
+    {
+        this->clear();
+    }
 
+    void clear()
+        {
+            VLOG(1) << "deleting elements...\n";
+            M_elements.clear();
+        }
     //@}
 
     /** @name Operator overloads
@@ -381,11 +393,11 @@ public:
 
     bool isBoundaryElement( element_type const & e ) const
     {
-        return M_elements.find( e )->isOnBoundary();
+        return elementIterator(e.id())->isOnBoundary();
     }
     bool isBoundaryElement( size_type const & id ) const
     {
-        return M_elements.find( element_type( id ) )->isOnBoundary();
+        return elementIterator(id)->isOnBoundary();
     }
 
     element_iterator elementIterator( size_type i ) const
@@ -393,17 +405,22 @@ public:
         return  M_elements.template get<0>().find( boost::make_tuple( this->worldCommElements().localRank(), i ) );
     };
 
-    element_iterator elementIterator( size_type i, size_type p ) const
+    element_iterator elementIterator( size_type i, rank_type p ) const
     {
         return  M_elements.template get<0>().find( boost::make_tuple( p, i ) );
     };
 
+    element_iterator elementIterator( element_type const& elt ) const
+    {
+        return elementIterator( elt.id(), elt.processId() );
+    };
+
     element_type const& element( size_type i ) const
     {
         return *M_elements.template get<0>().find( boost::make_tuple( this->worldCommElements().localRank(), i ) );
     };
 
-    element_type const& element( size_type i, size_type p ) const
+    element_type const& element( size_type i, rank_type p ) const
     {
         return *M_elements.template get<0>().find( boost::make_tuple( p, i ) );
     };
@@ -411,9 +428,10 @@ public:
     /**
      * \return \c true if element with id \p i is found, \c false otherwise
      */
-    bool hasElement( size_type i ) const
+    bool hasElement( size_type i, rank_type p = invalid_rank_type_value ) const
     {
-        return M_elements.template get<0>().find( boost::make_tuple( this->worldCommElements().localRank(), i ) ) !=
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommElements().localRank() : p;
+        return M_elements.template get<0>().find( boost::make_tuple( part, i ) ) !=
                M_elements.template get<0>().end();
     }
 
@@ -484,49 +502,55 @@ public:
     /**
      * \return the iterator \c begin over the elements with \c Marker1 \p m
      */
-    marker_element_const_iterator beginElementWithMarker( size_type m ) const
+    marker_element_const_iterator beginElementWithMarker( size_type m, rank_type p = invalid_rank_type_value ) const
     {
-        return M_elements.template get<detail::by_marker>().equal_range( boost::make_tuple( Marker1( m ), this->worldCommElements().localRank() ) ).first;
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommElements().localRank() : p;
+        return M_elements.template get<Feel::detail::by_marker>().equal_range( boost::make_tuple( Marker1( m ), part ) ).first;
     }
 
     /**
      * \return the iterator \c begin over the elements with \c Marker2 \p m
      */
-    marker2_element_const_iterator beginElementWithMarker2( size_type m ) const
+    marker2_element_const_iterator beginElementWithMarker2( size_type m, rank_type p = invalid_rank_type_value ) const
     {
-        return M_elements.template get<detail::by_marker2>().equal_range( boost::make_tuple( Marker2( m ), this->worldCommElements().localRank() ) ).first;
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommElements().localRank() : p;
+        return M_elements.template get<Feel::detail::by_marker2>().equal_range( boost::make_tuple( Marker2( m ), part ) ).first;
     }
 
     /**
      * \return the iterator \c begin over the elements with \c Marker3 \p m
      */
-    marker3_element_const_iterator beginElementWithMarker3( size_type m ) const
+    marker3_element_const_iterator beginElementWithMarker3( size_type m, rank_type p = invalid_rank_type_value ) const
     {
-        return M_elements.template get<detail::by_marker3>().equal_range( boost::make_tuple( Marker3( m ), this->worldCommElements().localRank() ) ).first;
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommElements().localRank() : p;
+        return M_elements.template get<Feel::detail::by_marker3>().equal_range( boost::make_tuple( Marker3( m ), part ) ).first;
     }
 
     /**
      * \return the iterator \c end over the elements with \c Marker1 \p m
      */
-    marker_element_const_iterator endElementWithMarker( size_type m ) const
+    marker_element_const_iterator endElementWithMarker( size_type m, rank_type p = invalid_rank_type_value ) const
     {
-        return M_elements.template get<detail::by_marker>().equal_range( boost::make_tuple( Marker1( m ), this->worldCommElements().localRank() ) ).second;
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommElements().localRank() : p;
+        return M_elements.template get<Feel::detail::by_marker>().equal_range( boost::make_tuple( Marker1( m ), part ) ).second;
     }
 
     /**
      * \return the iterator \c end over the elements with \c Marker2 \p m
      */
-    marker2_element_const_iterator endElementWithMarker2( size_type m ) const
+    marker2_element_const_iterator endElementWithMarker2( size_type m, rank_type p = invalid_rank_type_value ) const
     {
-        return M_elements.template get<detail::by_marker2>().equal_range( boost::make_tuple( Marker2( m ), this->worldCommElements().localRank() ) ).second;
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommElements().localRank() : p;
+        return M_elements.template get<Feel::detail::by_marker2>().equal_range( boost::make_tuple( Marker2( m ), part ) ).second;
     }
 
     /**
      * \return the iterator \c end over the elements with \c Marker3 \p m
      */
-    marker3_element_const_iterator endElementWithMarker3( size_type m ) const
+    marker3_element_const_iterator endElementWithMarker3( size_type m, rank_type p = invalid_rank_type_value ) const
     {
-        return M_elements.template get<detail::by_marker3>().equal_range( boost::make_tuple( Marker3( m ), this->worldCommElements().localRank() ) ).second;
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommElements().localRank() : p;
+        return M_elements.template get<Feel::detail::by_marker3>().equal_range( boost::make_tuple( Marker3( m ), part ) ).second;
     }
 
     /**
@@ -534,9 +558,10 @@ public:
      * with \c Marker1 \p m on processor \p p
      */
     std::pair<marker_element_const_iterator, marker_element_const_iterator>
-    elementsWithMarker( size_type m, size_type p ) const
+    elementsWithMarker( size_type m, rank_type p = invalid_rank_type_value ) const
     {
-        return M_elements.template get<detail::by_marker>().equal_range( boost::make_tuple( Marker1( m ), this->worldCommElements().localRank() ) );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommElements().localRank() : p;
+        return M_elements.template get<Feel::detail::by_marker>().equal_range( boost::make_tuple( Marker1( m ), part ) );
     }
 
 
@@ -545,9 +570,10 @@ public:
      * with \c Marker2 \p m on processor \p p
      */
     std::pair<marker2_element_const_iterator, marker2_element_const_iterator>
-    elementsWithMarker2( size_type m, size_type p ) const
+    elementsWithMarker2( size_type m, rank_type p = invalid_rank_type_value ) const
     {
-        return M_elements.template get<detail::by_marker2>().equal_range( boost::make_tuple( Marker2( m ), this->worldCommElements().localRank() ) );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommElements().localRank() : p;
+        return M_elements.template get<Feel::detail::by_marker2>().equal_range( boost::make_tuple( Marker2( m ), part ) );
     }
 
     /**
@@ -555,38 +581,45 @@ public:
      * with \c Marker3 \p m on processor \p p
      */
     std::pair<marker3_element_const_iterator, marker3_element_const_iterator>
-    elementsWithMarker3( size_type m, size_type p ) const
+    elementsWithMarker3( size_type m, rank_type p = invalid_rank_type_value ) const
     {
-        return M_elements.template get<detail::by_marker3>().equal_range( boost::make_tuple( Marker3( m ), this->worldCommElements().localRank() ) );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommElements().localRank() : p;
+        return M_elements.template get<Feel::detail::by_marker3>().equal_range( boost::make_tuple( Marker3( m ), part ) );
     }
 
-    element_iterator beginElementWithProcessId( size_type m )
+    element_iterator beginElementWithProcessId( rank_type p = invalid_rank_type_value )
     {
-        return M_elements.template get<0>().lower_bound( boost::make_tuple( m ) );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommElements().localRank() : p;
+        return M_elements.template get<0>().lower_bound( boost::make_tuple( part ) );
     }
-    element_const_iterator beginElementWithProcessId( size_type m ) const
+    element_const_iterator beginElementWithProcessId( rank_type p = invalid_rank_type_value ) const
     {
-        return M_elements.template get<0>().lower_bound( boost::make_tuple( m ) );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommElements().localRank() : p;
+        return M_elements.template get<0>().lower_bound( boost::make_tuple( part ) );
     }
-    element_iterator endElementWithProcessId( size_type m )
+    element_iterator endElementWithProcessId( rank_type p = invalid_rank_type_value )
     {
-        return M_elements.template get<0>().upper_bound( boost::make_tuple( m ) );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommElements().localRank() : p;
+        return M_elements.template get<0>().upper_bound( boost::make_tuple( part ) );
     }
-    element_const_iterator endElementWithProcessId( size_type m ) const
+    element_const_iterator endElementWithProcessId( rank_type p = invalid_rank_type_value ) const
     {
-        return M_elements.template get<0>().upper_bound( boost::make_tuple( m ) );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommElements().localRank() : p;
+        return M_elements.template get<0>().upper_bound( boost::make_tuple( part ) );
     }
 
     std::pair<element_const_iterator, element_const_iterator>
-    elementsWithProcessId( size_type m ) const
+    elementsWithProcessId( rank_type p = invalid_rank_type_value ) const
     {
-        return M_elements.template get<0>().equal_range( boost::make_tuple( m ) );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommElements().localRank() : p;
+        return M_elements.template get<0>().equal_range( boost::make_tuple( part ) );
     }
 
     std::pair<element_iterator, element_iterator>
-    elementsWithProcessId( size_type m )
+    elementsWithProcessId( rank_type p = invalid_rank_type_value )
     {
-        return M_elements.template get<0>().equal_range( boost::make_tuple( m ) );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommElements().localRank() : p;
+        return M_elements.template get<0>().equal_range( boost::make_tuple( part ) );
     }
 
     /**
@@ -622,7 +655,7 @@ public:
     marker_elements &
     elementsByMarker()
     {
-        return M_elements.template get<detail::by_marker>();
+        return M_elements.template get<Feel::detail::by_marker>();
     }
 
     /**
@@ -634,7 +667,7 @@ public:
     marker2_elements &
     elementsByMarker2()
     {
-        return M_elements.template get<detail::by_marker2>();
+        return M_elements.template get<Feel::detail::by_marker2>();
     }
 
     /**
@@ -646,7 +679,7 @@ public:
     marker3_elements &
     elementsByMarker3()
     {
-        return M_elements.template get<detail::by_marker3>();
+        return M_elements.template get<Feel::detail::by_marker3>();
     }
 
     /**
@@ -658,7 +691,7 @@ public:
     marker_elements const&
     elementsByMarker() const
     {
-        return M_elements.template get<detail::by_marker>();
+        return M_elements.template get<Feel::detail::by_marker>();
     }
 
     /**
@@ -670,7 +703,7 @@ public:
     marker2_elements const&
     elementsByMarker2() const
     {
-        return M_elements.template get<detail::by_marker2>();
+        return M_elements.template get<Feel::detail::by_marker2>();
     }
 
     /**
@@ -682,7 +715,7 @@ public:
     marker3_elements const&
     elementsByMarker3() const
     {
-        return M_elements.template get<detail::by_marker3>();
+        return M_elements.template get<Feel::detail::by_marker3>();
     }
 
     /**
@@ -694,7 +727,7 @@ public:
     pid_elements &
     elementsByProcessId()
     {
-        return M_elements.template get<detail::by_pid>();
+        return M_elements.template get<Feel::detail::by_pid>();
     }
 
     /**
@@ -706,7 +739,7 @@ public:
     pid_elements const&
     elementsByProcessId() const
     {
-        return M_elements.template get<detail::by_pid>();
+        return M_elements.template get<Feel::detail::by_pid>();
     }
 
     /**
@@ -714,10 +747,11 @@ public:
      *  element on processor \p p
      */
     std::pair<location_element_const_iterator, location_element_const_iterator>
-    boundaryElements( uint16_type entity_min_dim, uint16_type entity_max_dim, size_type p  ) const
+    boundaryElements( uint16_type entity_min_dim, uint16_type entity_max_dim, rank_type p = invalid_rank_type_value  ) const
     {
-        auto lower = M_elements.template get<detail::by_location>().lower_bound( boost::make_tuple( this->worldCommElements().localRank(), bool(ON_BOUNDARY), entity_min_dim ) );
-        auto upper = M_elements.template get<detail::by_location>().upper_bound( boost::make_tuple( this->worldCommElements().localRank(), bool(ON_BOUNDARY), entity_max_dim ) );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommElements().localRank() : p;
+        auto lower = M_elements.template get<Feel::detail::by_location>().lower_bound( boost::make_tuple( part, bool(ON_BOUNDARY), entity_min_dim ) );
+        auto upper = M_elements.template get<Feel::detail::by_location>().upper_bound( boost::make_tuple( part, bool(ON_BOUNDARY), entity_max_dim ) );
         return std::make_pair( lower, upper );
 
     }
@@ -727,12 +761,13 @@ public:
      *  element on processor \p p
      */
     std::pair<location_element_const_iterator, location_element_const_iterator>
-    boundaryElements( size_type p  ) const
+    boundaryElements( rank_type p = invalid_rank_type_value  ) const
     {
-        return boundaryElements( 0, 2, p );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommElements().localRank() : p;
+        return boundaryElements( 0, 2, part );
         //auto lower = boost::make_tuple( this->worldCommElements().localRank(), bool(ON_BOUNDARY), 0);
         //auto upper = boost::make_tuple( this->worldCommElements().localRank(), bool(ON_BOUNDARY), 2);
-        //return M_elements.template get<detail::by_location>().range( lower, upper );
+        //return M_elements.template get<Feel::detail::by_location>().range( lower, upper );
 
     }
 
@@ -742,9 +777,12 @@ public:
      *  element on processor \p p
      */
     std::pair<location_element_const_iterator, location_element_const_iterator>
-    internalElements( size_type p  ) const
+    internalElements( rank_type p = invalid_rank_type_value  ) const
     {
-        return M_elements.template get<detail::by_location>().equal_range( boost::make_tuple( this->worldCommElements().localRank(),  bool(INTERNAL), invalid_uint16_type_value ) );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommElements().localRank() : p;
+        auto lower = M_elements.template get<Feel::detail::by_location>().lower_bound( boost::make_tuple( part, bool(INTERNAL), 0 ) );
+        auto upper = M_elements.template get<Feel::detail::by_location>().upper_bound( boost::make_tuple( part, bool(INTERNAL), invalid_uint16_type_value ) );
+        return std::make_pair( lower, upper );
     }
 
 
@@ -757,7 +795,7 @@ public:
     location_elements &
     elementsByLocation()
     {
-        return M_elements.template get<detail::by_location>();
+        return M_elements.template get<Feel::detail::by_location>();
     }
 
     /**
@@ -769,7 +807,7 @@ public:
     location_elements const&
     elementsByLocation() const
     {
-        return M_elements.template get<detail::by_location>();
+        return M_elements.template get<Feel::detail::by_location>();
     }
 
     /**
@@ -777,18 +815,20 @@ public:
      *
      * @return the begin() iterator on all the internal elements
      */
-    location_element_iterator beginInternalElement()
+    location_element_iterator beginInternalElement( rank_type p = invalid_rank_type_value )
     {
-        return M_elements.template get<detail::by_location>().equal_range( boost::make_tuple( this->worldCommElements().localRank(). INTERNAL, invalid_uint16_type_value ) ).first;
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommElements().localRank() : p;
+        return M_elements.template get<Feel::detail::by_location>().equal_range( boost::make_tuple( part, INTERNAL, invalid_uint16_type_value ) ).first;
     }
     /**
      * get the end() iterator on all the internal elements
      *
      * @return the end() iterator on all the internal elements
      */
-    location_element_iterator endInternalElement()
+    location_element_iterator endInternalElement( rank_type p = invalid_rank_type_value )
     {
-        return M_elements.template get<detail::by_location>().equal_range( boost::make_tuple( this->worldCommElements().localRank(), INTERNAL, invalid_uint16_type_value ) ).second;
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommElements().localRank() : p;
+        return M_elements.template get<Feel::detail::by_location>().equal_range( boost::make_tuple( part, INTERNAL, invalid_uint16_type_value ) ).second;
     }
 
     /**
@@ -796,9 +836,10 @@ public:
      *
      * @return the begin() iterator on all the internal elements
      */
-    location_element_const_iterator beginInternalElement() const
+    location_element_const_iterator beginInternalElement( rank_type p = invalid_rank_type_value ) const
     {
-        return M_elements.template get<detail::by_location>().equal_range( boost::make_tuple( this->worldCommElements().localRank(), INTERNAL, invalid_uint16_type_value ) ).first;
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommElements().localRank() : p;
+        return M_elements.template get<Feel::detail::by_location>().equal_range( boost::make_tuple( part, INTERNAL, invalid_uint16_type_value ) ).first;
     }
 
     /**
@@ -806,9 +847,10 @@ public:
      *
      * @return the end() iterator on all the internal elements
      */
-    location_element_const_iterator endInternalElement() const
+    location_element_const_iterator endInternalElement( rank_type p = invalid_rank_type_value ) const
     {
-        return M_elements.template get<detail::by_location>().equal_range( boost::make_tuple( this->worldCommElements().localRank(), INTERNAL, invalid_uint16_type_value ) ).second;
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommElements().localRank() : p;
+        return M_elements.template get<Feel::detail::by_location>().equal_range( boost::make_tuple( part, INTERNAL, invalid_uint16_type_value ) ).second;
     }
 
     /**
@@ -816,18 +858,20 @@ public:
      *
      * @return the begin() iterator on all the boundary elements
      */
-    location_element_iterator beginElementOnBoundary()
+    location_element_iterator beginElementOnBoundary( rank_type p = invalid_rank_type_value )
     {
-        return M_elements.template get<detail::by_location>().lower_bound( boost::make_tuple( this->worldCommElements().localRank(), ON_BOUNDARY, 0 ) );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommElements().localRank() : p;
+        return M_elements.template get<Feel::detail::by_location>().lower_bound( boost::make_tuple( part, ON_BOUNDARY, 0 ) );
     }
     /**
      * get the end() iterator on all the boundary elements
      *
      * @return the end() iterator on all the boundary elements
      */
-    location_element_iterator endElementOnBoundary()
+    location_element_iterator endElementOnBoundary( rank_type p = invalid_rank_type_value )
     {
-        return M_elements.template get<detail::by_location>().upper_bound( boost::make_tuple( this->worldCommElements().localRank(), ON_BOUNDARY, 2 ) );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommElements().localRank() : p;
+        return M_elements.template get<Feel::detail::by_location>().upper_bound( boost::make_tuple( part, ON_BOUNDARY, 2 ) );
     }
 
     /**
@@ -835,9 +879,10 @@ public:
      *
      * @return the begin() iterator on all the boundary elements
      */
-    location_element_const_iterator beginElementOnBoundary() const
+    location_element_const_iterator beginElementOnBoundary( rank_type p = invalid_rank_type_value ) const
     {
-        return M_elements.template get<detail::by_location>().lower_bound( boost::make_tuple( this->worldCommElements().localRank(), ON_BOUNDARY, 0 ) );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommElements().localRank() : p;
+        return M_elements.template get<Feel::detail::by_location>().lower_bound( boost::make_tuple( part, ON_BOUNDARY, 0 ) );
     }
 
     /**
@@ -845,9 +890,10 @@ public:
      *
      * @return the end() iterator on all the boundary elements
      */
-    location_element_const_iterator endElementOnBoundary() const
+    location_element_const_iterator endElementOnBoundary( rank_type p = invalid_rank_type_value ) const
     {
-        return M_elements.template get<detail::by_location>().upper_bound( boost::make_tuple( this->worldCommElements().localRank(), ON_BOUNDARY, 2 ) );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommElements().localRank() : p;
+        return M_elements.template get<Feel::detail::by_location>().upper_bound( boost::make_tuple( part, ON_BOUNDARY, 2 ) );
     }
 
     /**
@@ -857,9 +903,9 @@ public:
      */
     ghostcell_element_iterator beginGhostElement()
     {
-        //return M_elements.template get<detail::by_ghostcell>().equal_range(boost::make_tuple(true)).first;
-        return M_elements.template get<detail::by_ghostcell>().equal_range( true ).first;
-        //return M_elements.template get<detail::by_ghostcell>().begin();
+        //return M_elements.template get<Feel::detail::by_ghostcell>().equal_range(boost::make_tuple(true)).first;
+        return M_elements.template get<Feel::detail::by_ghostcell>().equal_range( true ).first;
+        //return M_elements.template get<Feel::detail::by_ghostcell>().begin();
     }
 
     /**
@@ -869,9 +915,9 @@ public:
      */
     ghostcell_element_iterator endGhostElement()
     {
-        //return M_elements.template get<detail::by_ghostcell>().equal_range(boost::make_tuple(true)).second;
-        return M_elements.template get<detail::by_ghostcell>().equal_range( true ).second;
-        //return M_elements.template get<detail::by_ghostcell>().end();
+        //return M_elements.template get<Feel::detail::by_ghostcell>().equal_range(boost::make_tuple(true)).second;
+        return M_elements.template get<Feel::detail::by_ghostcell>().equal_range( true ).second;
+        //return M_elements.template get<Feel::detail::by_ghostcell>().end();
     }
 
     /**
@@ -881,9 +927,9 @@ public:
      */
     ghostcell_element_const_iterator beginGhostElement() const
     {
-        //return M_elements.template get<detail::by_ghostcell>().equal_range(boost::make_tuple(true)).first;
-        return M_elements.template get<detail::by_ghostcell>().equal_range( true ).first;
-        //return M_elements.template get<detail::by_ghostcell>().begin();
+        //return M_elements.template get<Feel::detail::by_ghostcell>().equal_range(boost::make_tuple(true)).first;
+        return M_elements.template get<Feel::detail::by_ghostcell>().equal_range( true ).first;
+        //return M_elements.template get<Feel::detail::by_ghostcell>().begin();
     }
 
     /**
@@ -893,9 +939,9 @@ public:
      */
     ghostcell_element_const_iterator endGhostElement() const
     {
-        //return M_elements.template get<detail::by_ghostcell>().equal_range(boost::make_tuple(true)).second;
-        return M_elements.template get<detail::by_ghostcell>().equal_range( true ).second;
-        //return M_elements.template get<detail::by_ghostcell>().end();
+        //return M_elements.template get<Feel::detail::by_ghostcell>().equal_range(boost::make_tuple(true)).second;
+        return M_elements.template get<Feel::detail::by_ghostcell>().equal_range( true ).second;
+        //return M_elements.template get<Feel::detail::by_ghostcell>().end();
     }
 
 
