@@ -85,7 +85,7 @@ public:
         multi_index::ordered_unique<
             multi_index::composite_key<face_type,
                                        multi_index::const_mem_fun<face_type,
-                                                                  uint16_type,
+                                                                  rank_type,
                                                                   &face_type::processId>,
                                        multi_index::const_mem_fun<face_type,
                                                                   size_type,
@@ -93,64 +93,67 @@ public:
 #endif
 
         // sort by less<int> on marker
-        multi_index::ordered_non_unique<multi_index::tag<detail::by_marker>,
+        multi_index::ordered_non_unique<multi_index::tag<Feel::detail::by_marker>,
                                         multi_index::composite_key<
                                             face_type,
                                             multi_index::const_mem_fun<face_type,
                                                                        Marker1 const&,
                                                                        &face_type::marker>,
                                             multi_index::const_mem_fun<face_type,
-                                                                       uint16_type,
+                                                                       rank_type,
                                                                        &face_type::processId>
                                             > >,
-        multi_index::ordered_non_unique<multi_index::tag<detail::by_marker2>,
+        multi_index::ordered_non_unique<multi_index::tag<Feel::detail::by_marker2>,
                                         multi_index::composite_key<
                                             face_type,
                                             multi_index::const_mem_fun<face_type,
                                                                        Marker2 const&,
                                                                        &face_type::marker2>,
                                             multi_index::const_mem_fun<face_type,
-                                                                       uint16_type,
+                                                                       rank_type,
                                                                        &face_type::processId>
                                             > >,
-        multi_index::ordered_non_unique<multi_index::tag<detail::by_marker3>,
+        multi_index::ordered_non_unique<multi_index::tag<Feel::detail::by_marker3>,
                                         multi_index::composite_key<
                                             face_type,
                                             multi_index::const_mem_fun<face_type,
                                                                        Marker3 const&,
                                                                        &face_type::marker3>,
                                             multi_index::const_mem_fun<face_type,
-                                                                       uint16_type,
+                                                                       rank_type,
                                                                        &face_type::processId>
                                             > >,
         // sort by less<int> on processId
-        multi_index::ordered_non_unique<multi_index::tag<detail::by_pid>,
+        multi_index::ordered_non_unique<multi_index::tag<Feel::detail::by_pid>,
                                         multi_index::const_mem_fun<face_type,
-                                                                   uint16_type,
+                                                                   rank_type,
                                                                    &face_type::processId> >,
 
 
         // sort by less<int> on boundary
-        multi_index::ordered_non_unique<multi_index::tag<detail::by_interprocessdomain>,
+        multi_index::ordered_non_unique<multi_index::tag<Feel::detail::by_interprocessdomain>,
                                         multi_index::composite_key<
                                             face_type,
                                             multi_index::const_mem_fun<face_type,
                                                                        bool,
                                                                        &face_type::isInterProcessDomain>,
                                             multi_index::const_mem_fun<face_type,
-                                                                       uint16_type,
-                                                                       &face_type::processId>
+                                                                       rank_type,
+                                                                       &face_type::partition1>,
+                                            multi_index::const_mem_fun<face_type,
+                                                                       rank_type,
+                                                                       &face_type::partition2>
                                             >
                                         >,
         // sort by less<int> on boundary
-        multi_index::ordered_non_unique<multi_index::tag<detail::by_location>,
+        multi_index::ordered_non_unique<multi_index::tag<Feel::detail::by_location>,
                                         multi_index::composite_key<
                                             face_type,
                                             multi_index::const_mem_fun<face_type,
                                                                        bool,
                                                                        &face_type::isOnBoundary>,
                                             multi_index::const_mem_fun<face_type,
-                                                                       uint16_type,
+                                                                       rank_type,
                                                                        &face_type::processId>
                                             >
                                         > >
@@ -159,9 +162,9 @@ public:
 
     typedef typename faces_type::iterator face_iterator;
     typedef typename faces_type::const_iterator face_const_iterator;
-    typedef typename faces_type::template index<detail::by_marker>::type marker_faces;
-    typedef typename faces_type::template index<detail::by_marker2>::type marker2_faces;
-    typedef typename faces_type::template index<detail::by_marker3>::type marker3_faces;
+    typedef typename faces_type::template index<Feel::detail::by_marker>::type marker_faces;
+    typedef typename faces_type::template index<Feel::detail::by_marker2>::type marker2_faces;
+    typedef typename faces_type::template index<Feel::detail::by_marker3>::type marker3_faces;
     typedef typename marker_faces::iterator marker_face_iterator;
     typedef typename marker_faces::const_iterator marker_face_const_iterator;
     typedef typename marker2_faces::iterator marker2_face_iterator;
@@ -169,17 +172,18 @@ public:
     typedef typename marker3_faces::iterator marker3_face_iterator;
     typedef typename marker3_faces::const_iterator marker3_face_const_iterator;
 
-    typedef typename faces_type::template index<detail::by_location>::type location_faces;
+    typedef typename faces_type::template index<Feel::detail::by_location>::type location_faces;
     typedef typename location_faces::iterator location_face_iterator;
     typedef typename location_faces::const_iterator location_face_const_iterator;
 
-    typedef typename faces_type::template index<detail::by_pid>::type pid_faces;
+    typedef typename faces_type::template index<Feel::detail::by_pid>::type pid_faces;
     typedef typename pid_faces::iterator pid_face_iterator;
     typedef typename pid_faces::const_iterator pid_face_const_iterator;
 
-    typedef typename faces_type::template index<detail::by_interprocessdomain>::type interprocess_faces;
+    typedef typename faces_type::template index<Feel::detail::by_interprocessdomain>::type interprocess_faces;
     typedef typename interprocess_faces::iterator interprocess_face_iterator;
     typedef typename interprocess_faces::const_iterator interprocess_face_const_iterator;
+
     //@}
 
     /**
@@ -235,8 +239,14 @@ public:
     {}
 
     virtual ~Faces()
-    {}
-
+    {
+        this->clear();
+    }
+    void clear()
+        {
+            VLOG(1) << "deleting faces...\n";
+            M_faces.clear();
+        }
     //@}
 
     /** @name Operator overloads
@@ -333,104 +343,104 @@ public:
 
     marker_face_iterator beginFaceWithMarker()
     {
-        return M_faces.template get<detail::by_marker>().begin();
+        return M_faces.template get<Feel::detail::by_marker>().begin();
     }
     marker_face_const_iterator beginFaceWithMarker() const
     {
-        return M_faces.template get<detail::by_marker>().begin();
+        return M_faces.template get<Feel::detail::by_marker>().begin();
     }
     marker_face_iterator endFaceWithMarker()
     {
-        return M_faces.template get<detail::by_marker>().end();
+        return M_faces.template get<Feel::detail::by_marker>().end();
     }
     marker_face_const_iterator endFaceWithMarker() const
     {
-        return M_faces.template get<detail::by_marker>().end();
+        return M_faces.template get<Feel::detail::by_marker>().end();
     }
 
     marker_face_iterator beginFaceWithMarker( size_type m )
     {
-        return M_faces.template get<detail::by_marker>().lower_bound( Marker1( m ) );
+        return M_faces.template get<Feel::detail::by_marker>().lower_bound( Marker1( m ) );
     }
     marker_face_const_iterator beginFaceWithMarker( size_type m ) const
     {
-        return M_faces.template get<detail::by_marker>().lower_bound( Marker1( m ) );
+        return M_faces.template get<Feel::detail::by_marker>().lower_bound( Marker1( m ) );
     }
     marker_face_iterator endFaceWithMarker( size_type m )
     {
-        return M_faces.template get<detail::by_marker>().upper_bound( Marker1( m ) );
+        return M_faces.template get<Feel::detail::by_marker>().upper_bound( Marker1( m ) );
     }
     marker_face_const_iterator endFaceWithMarker( size_type m ) const
     {
-        return M_faces.template get<detail::by_marker>().upper_bound( Marker1( m ) );
+        return M_faces.template get<Feel::detail::by_marker>().upper_bound( Marker1( m ) );
     }
 
     marker2_face_iterator beginFaceWithMarker2()
     {
-        return M_faces.template get<detail::by_marker2>().begin();
+        return M_faces.template get<Feel::detail::by_marker2>().begin();
     }
     marker2_face_const_iterator beginFaceWithMarker2() const
     {
-        return M_faces.template get<detail::by_marker2>().begin();
+        return M_faces.template get<Feel::detail::by_marker2>().begin();
     }
     marker2_face_iterator endFaceWithMarker2()
     {
-        return M_faces.template get<detail::by_marker2>().end();
+        return M_faces.template get<Feel::detail::by_marker2>().end();
     }
     marker2_face_const_iterator endFaceWithMarker2() const
     {
-        return M_faces.template get<detail::by_marker2>().end();
+        return M_faces.template get<Feel::detail::by_marker2>().end();
     }
 
     marker2_face_iterator beginFaceWithMarker2( size_type m )
     {
-        return M_faces.template get<detail::by_marker2>().lower_bound( Marker2( m ) );
+        return M_faces.template get<Feel::detail::by_marker2>().lower_bound( Marker2( m ) );
     }
     marker2_face_const_iterator beginFaceWithMarker2( size_type m ) const
     {
-        return M_faces.template get<detail::by_marker2>().lower_bound( Marker2( m ) );
+        return M_faces.template get<Feel::detail::by_marker2>().lower_bound( Marker2( m ) );
     }
     marker2_face_iterator endFaceWithMarker2( size_type m )
     {
-        return M_faces.template get<detail::by_marker2>().upper_bound( Marker2( m ) );
+        return M_faces.template get<Feel::detail::by_marker2>().upper_bound( Marker2( m ) );
     }
     marker2_face_const_iterator endFaceWithMarker2( size_type m ) const
     {
-        return M_faces.template get<detail::by_marker2>().upper_bound( Marker2( m ) );
+        return M_faces.template get<Feel::detail::by_marker2>().upper_bound( Marker2( m ) );
     }
 
     marker3_face_iterator beginFaceWithMarker3()
     {
-        return M_faces.template get<detail::by_marker3>().begin();
+        return M_faces.template get<Feel::detail::by_marker3>().begin();
     }
     marker3_face_const_iterator beginFaceWithMarker3() const
     {
-        return M_faces.template get<detail::by_marker3>().begin();
+        return M_faces.template get<Feel::detail::by_marker3>().begin();
     }
     marker3_face_iterator endFaceWithMarker3()
     {
-        return M_faces.template get<detail::by_marker3>().end();
+        return M_faces.template get<Feel::detail::by_marker3>().end();
     }
     marker3_face_const_iterator endFaceWithMarker3() const
     {
-        return M_faces.template get<detail::by_marker3>().end();
+        return M_faces.template get<Feel::detail::by_marker3>().end();
     }
 
     marker3_face_iterator beginFaceWithMarker3( size_type m )
     {
-        return M_faces.template get<detail::by_marker3>().lower_bound( Marker3( m ) );
+        return M_faces.template get<Feel::detail::by_marker3>().lower_bound( Marker3( m ) );
     }
     marker3_face_const_iterator beginFaceWithMarker3( size_type m ) const
     {
-        return M_faces.template get<detail::by_marker3>().lower_bound( Marker3( m ) );
+        return M_faces.template get<Feel::detail::by_marker3>().lower_bound( Marker3( m ) );
     }
     marker3_face_iterator endFaceWithMarker3( size_type m )
     {
-        return M_faces.template get<detail::by_marker3>().upper_bound( Marker3( m ) );
+        return M_faces.template get<Feel::detail::by_marker3>().upper_bound( Marker3( m ) );
     }
     marker3_face_const_iterator endFaceWithMarker3( size_type m ) const
     {
-        return M_faces.template get<detail::by_marker3>().upper_bound( Marker3( m ) );
+        return M_faces.template get<Feel::detail::by_marker3>().upper_bound( Marker3( m ) );
     }
 
     face_iterator beginFaceWithId( size_type m )
@@ -455,21 +465,24 @@ public:
      * with marker \p m on processor \p p
      */
     std::pair<marker_face_iterator, marker_face_iterator>
-    facesWithMarker( size_type m, size_type p ) const
+    facesWithMarker( size_type m, rank_type p = invalid_rank_type_value ) const
     {
-        return M_faces.template get<detail::by_marker>().equal_range( boost::make_tuple( Marker1( m ), p ) );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommFaces().localRank() : p;
+        return M_faces.template get<Feel::detail::by_marker>().equal_range( boost::make_tuple( Marker1( m ), part ) );
     }
 
     std::pair<marker2_face_iterator, marker2_face_iterator>
-    facesWithMarker2( size_type m, size_type p ) const
+    facesWithMarker2( size_type m, rank_type p = invalid_rank_type_value ) const
     {
-        return M_faces.template get<detail::by_marker2>().equal_range( boost::make_tuple( Marker2( m ), p ) );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommFaces().localRank() : p;
+        return M_faces.template get<Feel::detail::by_marker2>().equal_range( boost::make_tuple( Marker2( m ), part ) );
     }
 
     std::pair<marker3_face_iterator, marker3_face_iterator>
-    facesWithMarker3( size_type m, size_type p ) const
+    facesWithMarker3( size_type m, rank_type p = invalid_rank_type_value ) const
     {
-        return M_faces.template get<detail::by_marker3>().equal_range( boost::make_tuple( Marker3( m ), p ) );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommFaces().localRank() : p;
+        return M_faces.template get<Feel::detail::by_marker3>().equal_range( boost::make_tuple( Marker3( m ), part ) );
     }
 
 
@@ -480,7 +493,7 @@ public:
     std::pair<location_face_iterator, location_face_iterator>
     facesOnBoundary() const
     {
-        return M_faces.template get<detail::by_location>().equal_range( boost::make_tuple( ON_BOUNDARY ) );
+        return M_faces.template get<Feel::detail::by_location>().equal_range( boost::make_tuple( ON_BOUNDARY ) );
     }
 
     /**
@@ -488,9 +501,9 @@ public:
      *  faces on processor \p p
      */
     std::pair<location_face_iterator, location_face_iterator>
-    facesOnBoundary( size_type p  ) const
+    facesOnBoundary( rank_type p  ) const
     {
-        return M_faces.template get<detail::by_location>().equal_range( boost::make_tuple( ON_BOUNDARY, p ) );
+        return M_faces.template get<Feel::detail::by_location>().equal_range( boost::make_tuple( ON_BOUNDARY, p ) );
     }
 
     /**
@@ -498,9 +511,10 @@ public:
      * on processor \p p
      */
     std::pair<location_face_iterator, location_face_iterator>
-    internalFaces() const
+    internalFaces( rank_type p = invalid_rank_type_value ) const
     {
-        return M_faces.template get<detail::by_location>().equal_range( boost::make_tuple( INTERNAL, this->worldCommFaces().localRank() ) );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommFaces().localRank() : p;
+        return M_faces.template get<Feel::detail::by_location>().equal_range( boost::make_tuple( INTERNAL, part ) );
     }
 
     /**
@@ -508,9 +522,13 @@ public:
      * on processor \p p
      */
     std::pair<interprocess_face_iterator, interprocess_face_iterator>
-    interProcessFaces() const
+    interProcessFaces( rank_type p = invalid_rank_type_value ) const
     {
-        return M_faces.template get<detail::by_interprocessdomain>().equal_range( boost::make_tuple( true, this->worldCommFaces().localRank() ) );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommFaces().localRank() : p;
+        if ( p != invalid_rank_type_value )
+            return M_faces.template get<Feel::detail::by_interprocessdomain>().equal_range( boost::make_tuple( true, part, p ) );
+        else
+            return M_faces.template get<Feel::detail::by_interprocessdomain>().equal_range( boost::make_tuple( true, part ) );
     }
 
 #if 0
@@ -519,9 +537,10 @@ public:
      * on processor \p p
      */
     std::pair<interprocess_face_iterator, interprocess_face_iterator>
-    intraProcessFaces() const
+    intraProcessFaces( rank_type p = invalid_rank_type_value ) const
     {
-        return M_faces.template get<detail::by_interprocessdomain>().equal_range( boost::make_tuple( false, this->worldCommFaces().localRank() ) );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommFaces().localRank() : p;
+        return M_faces.template get<Feel::detail::by_interprocessdomain>().equal_range( boost::make_tuple( false, part ) );
     }
 #endif
 
@@ -530,13 +549,11 @@ public:
      * on processor \p p
      */
     std::pair<pid_face_iterator, pid_face_iterator>
-    //std::pair<face_iterator, face_iterator>
-    facesWithProcessId( size_type p ) const
+    facesWithProcessId( rank_type p = invalid_rank_type_value ) const
     {
-        return M_faces.template get<detail::by_pid>().equal_range( p );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommFaces().localRank() : p;
+        return M_faces.template get<Feel::detail::by_pid>().equal_range( part );
     }
-    //{ return M_faces.template get<detail::by_pid>().equal_range( boost::make_tuple(/*this->worldCommFaces().localRank()*/p) ); }
-    //{ return M_faces.template get<0>().equal_range( boost::make_tuple(/*this->worldCommFaces().localRank()*/p) ); }
 
     /**
      * get the faces container by id
@@ -571,7 +588,7 @@ public:
     marker_faces &
     facesByMarker()
     {
-        return M_faces.template get<detail::by_marker>();
+        return M_faces.template get<Feel::detail::by_marker>();
     }
 
     /**
@@ -583,7 +600,7 @@ public:
     marker_faces const&
     facesByMarker() const
     {
-        return M_faces.template get<detail::by_marker>();
+        return M_faces.template get<Feel::detail::by_marker>();
     }
 
     /**
@@ -595,7 +612,7 @@ public:
     marker2_faces &
     facesByMarker2()
     {
-        return M_faces.template get<detail::by_marker2>();
+        return M_faces.template get<Feel::detail::by_marker2>();
     }
 
     /**
@@ -607,7 +624,7 @@ public:
     marker2_faces const&
     facesByMarker2() const
     {
-        return M_faces.template get<detail::by_marker2>();
+        return M_faces.template get<Feel::detail::by_marker2>();
     }
 
     /**
@@ -619,7 +636,7 @@ public:
     marker3_faces &
     facesByMarker3()
     {
-        return M_faces.template get<detail::by_marker3>();
+        return M_faces.template get<Feel::detail::by_marker3>();
     }
 
     /**
@@ -631,7 +648,7 @@ public:
     marker3_faces const&
     facesByMarker3() const
     {
-        return M_faces.template get<detail::by_marker3>();
+        return M_faces.template get<Feel::detail::by_marker3>();
     }
 
 
@@ -644,7 +661,7 @@ public:
     location_faces &
     facesByLocation()
     {
-        return M_faces.template get<detail::by_location>();
+        return M_faces.template get<Feel::detail::by_location>();
     }
 
     /**
@@ -656,7 +673,7 @@ public:
     location_faces const&
     facesByLocation() const
     {
-        return M_faces.template get<detail::by_location>();
+        return M_faces.template get<Feel::detail::by_location>();
     }
 
     /**
@@ -664,18 +681,20 @@ public:
      *
      * @return the begin() iterator on all the internal faces
      */
-    location_face_iterator beginInternalFace()
+    location_face_iterator beginInternalFace( rank_type p = invalid_rank_type_value )
     {
-        return M_faces.template get<detail::by_location>().lower_bound( boost::make_tuple( INTERNAL, this->worldCommFaces().localRank() ) );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommFaces().localRank() : p;
+        return M_faces.template get<Feel::detail::by_location>().lower_bound( boost::make_tuple( INTERNAL, part ) );
     }
     /**
      * get the end() iterator on all the internal faces
      *
      * @return the end() iterator on all the internal faces
      */
-    location_face_iterator endInternalFace()
+    location_face_iterator endInternalFace( rank_type p = invalid_rank_type_value )
     {
-        return M_faces.template get<detail::by_location>().upper_bound( boost::make_tuple( INTERNAL, this->worldCommFaces().localRank() ) );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommFaces().localRank() : p;
+        return M_faces.template get<Feel::detail::by_location>().upper_bound( boost::make_tuple( INTERNAL, part ) );
     }
 
     /**
@@ -683,9 +702,10 @@ public:
      *
      * @return the begin() iterator on all the internal faces
      */
-    location_face_const_iterator beginInternalFace() const
+    location_face_const_iterator beginInternalFace( rank_type p = invalid_rank_type_value ) const
     {
-        return M_faces.template get<detail::by_location>().lower_bound( boost::make_tuple( INTERNAL, this->worldCommFaces().localRank() ) );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommFaces().localRank() : p;
+        return M_faces.template get<Feel::detail::by_location>().lower_bound( boost::make_tuple( INTERNAL, part ) );
     }
 
     /**
@@ -693,9 +713,10 @@ public:
      *
      * @return the end() iterator on all the internal faces
      */
-    location_face_const_iterator endInternalFace() const
+    location_face_const_iterator endInternalFace( rank_type p = invalid_rank_type_value ) const
     {
-        return M_faces.template get<detail::by_location>().upper_bound( boost::make_tuple( INTERNAL, this->worldCommFaces().localRank() ) );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommFaces().localRank() : p;
+        return M_faces.template get<Feel::detail::by_location>().upper_bound( boost::make_tuple( INTERNAL, part ) );
     }
 
     /**
@@ -703,18 +724,20 @@ public:
      *
      * @return the begin() iterator on all the boundary faces
      */
-    location_face_iterator beginFaceOnBoundary()
+    location_face_iterator beginFaceOnBoundary( rank_type p = invalid_rank_type_value )
     {
-        return M_faces.template get<detail::by_location>().lower_bound( boost::make_tuple( ON_BOUNDARY, this->worldCommFaces().localRank() ) );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommFaces().localRank() : p;
+        return M_faces.template get<Feel::detail::by_location>().lower_bound( boost::make_tuple( ON_BOUNDARY, part ) );
     }
     /**
      * get the end() iterator on all the boundary faces
      *
      * @return the end() iterator on all the boundary faces
      */
-    location_face_iterator endFaceOnBoundary()
+    location_face_iterator endFaceOnBoundary( rank_type p = invalid_rank_type_value )
     {
-        return M_faces.template get<detail::by_location>().upper_bound( boost::make_tuple( ON_BOUNDARY, this->worldCommFaces().localRank() ) );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommFaces().localRank() : p;
+        return M_faces.template get<Feel::detail::by_location>().upper_bound( boost::make_tuple( ON_BOUNDARY, part ) );
     }
 
     /**
@@ -722,9 +745,10 @@ public:
      *
      * @return the begin() iterator on all the boundary faces
      */
-    location_face_const_iterator beginFaceOnBoundary() const
+    location_face_const_iterator beginFaceOnBoundary( rank_type p = invalid_rank_type_value ) const
     {
-        return M_faces.template get<detail::by_location>().lower_bound( boost::make_tuple( ON_BOUNDARY, this->worldCommFaces().localRank() ) );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommFaces().localRank() : p;
+        return M_faces.template get<Feel::detail::by_location>().lower_bound( boost::make_tuple( ON_BOUNDARY, part ) );
     }
 
     /**
@@ -732,9 +756,10 @@ public:
      *
      * @return the end() iterator on all the boundary faces
      */
-    location_face_const_iterator endFaceOnBoundary() const
+    location_face_const_iterator endFaceOnBoundary( rank_type p = invalid_rank_type_value ) const
     {
-        return M_faces.template get<detail::by_location>().upper_bound( boost::make_tuple( ON_BOUNDARY, this->worldCommFaces().localRank() ) );
+        const rank_type part = (p==invalid_rank_type_value)? this->worldCommFaces().localRank() : p;
+        return M_faces.template get<Feel::detail::by_location>().upper_bound( boost::make_tuple( ON_BOUNDARY, part ) );
     }
 
     //@}
@@ -800,13 +825,15 @@ public:
 
             if ( ( tag2_0 != -1 && tag2_0 == tag2_1 ) || e.isOnBoundary() )
                 e.setMarker2( tag2_0 );
-
+            else if ( tag2_0 != -1 && tag2_1 != -1 )
+                e.setMarker2( std::max(tag2_0,tag2_1) );
             else
-                e.setMarker3( 0 );
+                e.setMarker2( 0 );
 
             if ( ( tag3_0 != -1 && tag3_0 == tag3_1 ) || e.isOnBoundary() )
                 e.setMarker3( tag3_0 );
-
+            else if ( tag3_0 != -1 && tag3_1 != -1 )
+                e.setMarker3( std::max(tag3_0,tag3_1) );
             else
                 e.setMarker3( 0 );
         } );
