@@ -124,6 +124,9 @@ public:
     typedef typename GetGm<1>::type gm1_type;
     typedef typename GetGm<1>::ptrtype gm1_ptrtype;
 
+    typedef typename gm_type::super::reference_convex_type reference_convex_type;
+    typedef typename gm1_type::super::reference_convex_type reference_convex1_type;
+
     typedef typename super::vertex_permutation_type vertex_permutation_type;
     typedef typename super::edge_permutation_type edge_permutation_type;
     typedef typename super::face_permutation_type face_permutation_type;
@@ -152,7 +155,8 @@ public:
         M_measurefaces( numTopologicalFaces ),
         M_normals( nRealDim, numTopologicalFaces ),
         M_has_points( false ),
-        M_neighbors( numNeighbors, std::make_pair( invalid_size_type_value, 0 ) ),
+        M_neighbors( numNeighbors, std::make_pair( invalid_size_type_value, invalid_rank_type_value ) ),
+        M_meas_pneighbors( 0 ),
         M_marker1(),
         M_marker2(),
         M_marker3(),
@@ -182,7 +186,8 @@ public:
         M_measurefaces( numTopologicalFaces ),
         M_normals( nRealDim, numTopologicalFaces ),
         M_has_points( false ),
-        M_neighbors( numNeighbors, std::make_pair( invalid_size_type_value, 0 ) ),
+        M_neighbors( numNeighbors, std::make_pair( invalid_size_type_value, invalid_rank_type_value ) ),
+        M_meas_pneighbors( 0 ),
         M_marker1(),
         M_marker2(),
         M_marker3(),
@@ -206,7 +211,8 @@ public:
         M_measurefaces( numTopologicalFaces  ),
         M_normals( e.M_normals ),
         M_has_points( false ),
-        M_neighbors( numNeighbors, std::make_pair( invalid_size_type_value, 0 ) ),
+        M_neighbors( numNeighbors, std::make_pair( invalid_size_type_value, invalid_rank_type_value ) ),
+        M_meas_pneighbors( e.M_meas_pneighbors ),
         M_marker1( e.M_marker1 ),
         M_marker2( e.M_marker2 ),
         M_marker3( e.M_marker3 ),
@@ -224,7 +230,6 @@ public:
      */
     ~GeoND()
     {
-
     }
 
 #if 0
@@ -348,7 +353,7 @@ public:
      *
      * \return the pair neighbor \p n index and process \p id it belongs to
      */
-    std::pair<size_type,size_type> const& neighbor( uint16_type n ) const
+    std::pair<size_type,rank_type> const& neighbor( uint16_type n ) const
     {
         return M_neighbors[n];
     }
@@ -356,7 +361,7 @@ public:
     /**
      * set the \p n -th neighbor with \p neigh
      */
-    void setNeighbor( uint16_type n, size_type neigh_id, size_type proc_id )
+    void setNeighbor( uint16_type n, size_type neigh_id, rank_type proc_id )
     {
         M_neighbors[n] = std::make_pair( neigh_id, proc_id );
     }
@@ -456,7 +461,7 @@ public:
      */
     PointType & reversepoint( uint16_type const i )
     {
-        return *( static_cast<POINTTYPE *>( M_points[ detail::ReversePoint<GEOSHAPE>::operate( i ) ] ) );
+        return *( static_cast<POINTTYPE *>( M_points[ Feel::detail::ReversePoint<GEOSHAPE>::operate( i ) ] ) );
     }
 
 
@@ -470,7 +475,7 @@ public:
      */
     PointType const & reversepoint ( uint16_type const i ) const
     {
-        return *( static_cast<POINTTYPE *>( M_points[ detail::ReversePoint<GEOSHAPE>::operate( i ) ] ) );
+        return *( static_cast<POINTTYPE *>( M_points[ Feel::detail::ReversePoint<GEOSHAPE>::operate( i ) ] ) );
     }
 
 
@@ -739,7 +744,8 @@ public:
 
             if ( tags[2] > 1 )
             {
-                std::vector<int> p( tags[2]-1 );
+                // ghosts
+                std::vector<rank_type> p( tags[2]-1 );
 
                 for ( size_type i = 0; i < p.size(); ++i )
                 {
@@ -870,7 +876,7 @@ private:
     /**
      * store neighbor element id
      */
-    std::vector<std::pair<size_type,size_type> > M_neighbors;
+    std::vector<std::pair<size_type,rank_type> > M_neighbors;
     //! point element neighbors
     std::set<size_type> M_pneighbors;
     //! measure of the set of point element neighbors
@@ -919,7 +925,7 @@ GeoND<Dim,GEOSHAPE, T, POINTTYPE>::showMe( bool verbose, std::ostream & out ) co
     for ( int i = 0; i < numVertices; i++ )
     {
         out << "POINT id = " << i << std::endl;
-        out << point( i ).showMe( verbose, out );
+        point( i ).showMe( verbose, out );
     }
 
     out << "----- END OF GeoND data ---" << std::endl << std::endl;
