@@ -29,6 +29,8 @@
 #include <boost/none_t.hpp>
 #endif /* BOOST_VERSION >= 103400 */
 
+#include <Eigen/Core>
+
 #include <boost/mpl/vector.hpp>
 #include <boost/shared_ptr.hpp>
 #include <boost/optional.hpp>
@@ -99,13 +101,15 @@ template<uint16_type Dim,
          template<class, uint16_type, class> class Pts, uint16_type > class PP = Lagrange>
 class GeoMap
     :
-    public PP<Order,Scalar, Continuous,PointSetEquiSpaced, 0>::template apply<Dim,RealDim/*Dim*/, T, Entity<Dim,Order,/*RealDim*/Dim> >::result_type,
-       public boost::enable_shared_from_this<GeoMap<Dim, Order, RealDim, T, Entity, PP > >
+    public PP<Order,Scalar, Continuous,PointSetEquiSpaced, 0>::template apply<Dim,RealDim/*Dim*/, T, Entity<Dim,Order,/*RealDim*/Dim> >::result_type//,
+//public boost::enable_shared_from_this<GeoMap<Dim, Order, RealDim, T, Entity, PP > >
        //public PP<Order,Scalar, PointSetFekete>::template apply<Dim, T, Entity<Dim,Order,Dim> >::result_type
        {
            //typedef typename PP<Order, Scalar, PointSetFekete>::template apply<Dim, T, Entity<Dim,Order,Dim> >::result_type super;
            typedef typename PP<Order, Scalar, Continuous, PointSetEquiSpaced, 0>::template apply<Dim, RealDim/*Dim*/, T, Entity<Dim,Order,/*RealDim*/Dim> >::result_type super;
 
+           //typedef boost::enable_shared_from_this<GeoMap<Dim, Order, RealDim, T, Entity, PP > > super_enable_this;
+
            static const uint16_type nRealDimCheck2d = mpl::if_< mpl::less_equal<mpl::int_<2>,mpl::int_<RealDim> >,
            mpl::int_<RealDim>,
            mpl::int_<Dim> >::type::value;
@@ -400,13 +404,11 @@ void transform( matrix_node_t_type const& G,
                 precompute_type const* pc,
                 matrix_type & x ) const
 {
-#if 0
-    blas::gemm( traits::NO_TRANSPOSE, traits::NO_TRANSPOSE,
-                1.0, G, pc->phi(),
-                0.0, x );
-#else
-    ublas::axpy_prod( G, pc->phi(), x, true );
-#endif
+        Eigen::Map<Eigen::Matrix<value_type,Eigen::Dynamic,Eigen::Dynamic,Eigen::RowMajor>> P( G.data(), G.rows(), G.cols() );
+        Eigen::Map<Eigen::Matrix<value_type,Eigen::Dynamic,Eigen::Dynamic,Eigen::RowMajor>> Phi( pc->phi().data(), pc->phi().rows(), pc->phi().cols() );
+        Eigen::Map<Eigen::Matrix<value_type,Eigen::Dynamic,Eigen::Dynamic,Eigen::RowMajor>> X( x.data(), x.size() );
+        X  = P*Phi;
+    //ublas::axpy_prod( G, pc->phi(), x, true );
 }
 
 
@@ -689,6 +691,7 @@ class Context
         M_CS( PDim, PDim ),
         M_CSi( PDim, PDim ),
         M_B( NDim, PDim ),
+        M_Ptangent( NDim, NDim ),
         M_B3( boost::extents[NDim][NDim][PDim][PDim] ),
         M_id( __e.id() ),
         M_e_marker( __e.marker() ),
@@ -754,6 +757,7 @@ Context( gm_ptrtype __gm,
     M_CS( PDim, PDim ),
     M_CSi( PDim, PDim ),
     M_B( NDim, PDim ),
+    M_Ptangent( NDim, NDim ),
     M_B3( boost::extents[NDim][NDim][PDim][PDim] ),
     M_id( __e.id() ),
     M_e_marker( __e.marker() ),
@@ -813,6 +817,7 @@ Context( gmc_ptrtype& p )
     M_CS( p->M_CS ),
     M_CSi( p->M_CSi ),
     M_B( p->M_B ),
+    M_Ptangent( p->M_Ptangent ),
     M_B3( p->M_B3 ),
     M_id( p->M_id ),
     M_e_marker( p->M_e_marker ),
@@ -915,7 +920,7 @@ void update( element_type const& __e, uint16_type __f )
 
 }
 
-void update( element_type const& __e, uint16_type __f, permutation_type __perm )
+void update( element_type const& __e, uint16_type __f, permutation_type __perm, bool __updateJacobianCtx=true )
 {
     //M_element_c = boost::shared_ptr<element_type const>(&__e);
     M_element = boost::addressof( __e );
@@ -958,7 +963,7 @@ void update( element_type const& __e, uint16_type __f, permutation_type __perm )
             }
     }
 
-    if ( vm::has_jacobian<context>::value )
+    if ( vm::has_jacobian<context>::value && __updateJacobianCtx )
     {
         updateJKBN( mpl::bool_<is_linear>() );
     }
@@ -1208,6 +1213,15 @@ value_type B( int c1, int c2, int q ) const
     return B( q, mpl::bool_<is_linear>() )( c1, c2 );
 }
 
+matrix_type const& projectorTangent( int i ) const
+{
+    return M_Ptangent;
+}
+value_type projectorTangent( int c1, int c2, int q ) const
+{
+    return M_Ptangent(c1,c2);
+}
+
 matrix_type const& K( int i ) const
 {
     return M_K;
@@ -1626,74 +1640,60 @@ permutation_type permutation( mpl::bool_<true> ) const
     return M_perm;
 }
 
+void updateJKBN( mpl::bool_<true>, mpl::true_  )
+{
+        Eigen::Map<Eigen::Matrix<value_type,Eigen::Dynamic,Eigen::Dynamic,Eigen::RowMajor>> P( M_G.data().begin(), M_G.size1(), M_G.size2() );
+        Eigen::Map<Eigen::Matrix<value_type,Eigen::Dynamic,Eigen::Dynamic,Eigen::RowMajor>> GradPhi( M_g_linear.data().begin(), M_g_linear.size1(), M_g_linear.size2() );
+        Eigen::Map<Eigen::Matrix<value_type,NDim,PDim,((PDim==1)?Eigen::ColMajor:Eigen::RowMajor)>> MK( M_K.data().begin(), M_K.size1(), M_K.size2() );
+        Eigen::Map<Eigen::Matrix<value_type,PDim,PDim,((NDim==1)?Eigen::ColMajor:Eigen::RowMajor)>> MCS( M_CS.data().begin(), M_CS.size1(), M_CS.size2() );
+        Eigen::Map<Eigen::Matrix<value_type,PDim,PDim,((PDim==1)?Eigen::ColMajor:Eigen::RowMajor)>> MCSi( M_CSi.data().begin(), M_CSi.size1(), M_CSi.size2() );
+        Eigen::Map<Eigen::Matrix<value_type,NDim,PDim,((PDim==1)?Eigen::ColMajor:Eigen::RowMajor)>> MB( M_B.data().begin(), M_B.size1(), M_B.size2() );
+        MK.noalias() =  P*GradPhi;
+        M_J = math::abs( MK.determinant() );
+        MCS = MK.inverse();
+        MB.noalias() = MCS.transpose();
+
+}
+void updateJKBN( mpl::bool_<true>, mpl::false_  )
+{
+#if 0
+        Eigen::Map<Eigen::Matrix<value_type,Eigen::Dynamic,Eigen::Dynamic,Eigen::ColMajor>> P( M_G.data().begin(), M_G.size1(), M_G.size2() );
+        Eigen::Map<Eigen::Matrix<value_type,Eigen::Dynamic,Eigen::Dynamic,Eigen::RowMajor>> GradPhi( M_g_linear.data().begin(), M_g_linear.size1(), M_g_linear.size2() );
+        Eigen::Map<Eigen::Matrix<value_type,NDim,PDim,((PDim==1)?Eigen::ColMajor:Eigen::RowMajor)>> MK( M_K.data().begin(), M_K.size1(), M_K.size2() );
+        Eigen::Map<Eigen::Matrix<value_type,PDim,PDim,((NDim==1)?Eigen::ColMajor:Eigen::RowMajor)>> MCS( M_CS.data().begin(), M_CS.size1(), M_CS.size2() );
+        Eigen::Map<Eigen::Matrix<value_type,PDim,PDim,((PDim==1)?Eigen::ColMajor:Eigen::RowMajor)>> MCSi( M_CSi.data().begin(), M_CSi.size1(), M_CSi.size2() );
+        Eigen::Map<Eigen::Matrix<value_type,NDim,PDim,((PDim==1)?Eigen::ColMajor:Eigen::RowMajor)>> MB( M_B.data().begin(), M_B.size1(), M_B.size2() );
+        MK.noalias() =  P*GradPhi;
+        //ublas::axpy_prod( M_G, M_g_linear, M_K, true );
+
+        // CS = K^T K
+        MCSi.noalias() = MK.transpose()*MK;
+        M_J = math::sqrt( math::abs( MCSi.determinant() ) );
+        //if ( vm::has_kb<context>::value )
+        {
+                MCS=MCSi.inverse();
+                // B = K CS
+                MB.noalias() = MK*MCS;
+        }
+#else
+        ublas::axpy_prod( M_G, M_g_linear, M_K, true );
+        ublas::noalias( M_CS ) = ublas::prod( ublas::trans( M_K ), M_K );
+        M_J = math::sqrt( math::abs( det<PDim>( M_CS ) ) );
+        inverse<PDim>( M_CS, M_CSi );
+        ublas::axpy_prod( M_K, M_CSi, M_B, true );
+
+#endif
+
+}
 /**
  * update Jacobian data : linear case
  */
 void updateJKBN( mpl::bool_<true>  )
 {
-
     if ( !M_gm->isCached() ||
             ( M_gm->isCached() && M_gm->cached( M_id ) == false ) )
     {
-#if 0
-
-        if ( boost::is_arithmetic<value_type>::value )
-            atlas::gemm( traits::NO_TRANSPOSE, traits::NO_TRANSPOSE,
-                         1.0, M_G, M_g_linear,
-                         0.0, M_K );
-
-        else
-#endif
-            ublas::axpy_prod( M_G, M_g_linear, M_K, true );
-
-        if ( NDim == PDim )
-        {
-            M_J = math::abs( det<NDim>( M_K ) );
-            //if ( vm::has_kb<context>::value )
-            {
-#if 0
-                inverse<NDim>( M_K, M_CS, M_J );
-                ublas::noalias( M_B ) = ublas::trans( M_CS );
-#else
-                inverse<NDim>( M_K, M_CS );
-                ublas::noalias( M_B ) = ublas::trans( M_CS );
-#endif
-            }
-        }
-
-        else // N != P
-        {
-            // CS = K^T K
-#if 0
-            if ( boost::is_arithmetic<value_type>::value )
-                atlas::gemm( traits::TRANSPOSE, traits::NO_TRANSPOSE,
-                             1.0, M_K, M_K,
-                             0.0, M_CS );
-
-            else
-#endif
-
-                ublas::noalias( M_CS ) = ublas::prod( ublas::trans( M_K ), M_K );
-
-            M_J = math::sqrt( math::abs( det<PDim>( M_CS ) ) );
-            //if ( vm::has_kb<context>::value )
-            {
-                inverse<PDim>( M_CS, M_CSi );
-                // B = K CS
-#if 0
-
-                if ( boost::is_arithmetic<value_type>::value )
-                    atlas::gemm( traits::NO_TRANSPOSE, traits::NO_TRANSPOSE,
-                                 1.0, M_K, M_CSi,
-                                 0.0, M_B );
-
-                else
-#endif
-                    ublas::axpy_prod( M_K, M_CSi, M_B, true );
-            }
-
-        }
-
+        updateJKBN( mpl::true_(), mpl::bool_<NDim==PDim>() );
         if ( M_gm->isCached() )
         {
             // cache J, K and B
@@ -1749,16 +1749,10 @@ void updateJKBN( mpl::bool_<true>  )
 
     if ( ( ( NDim != PDim ) || ( vm::has_normal<context>::value ) ) && ( M_face_id != invalid_uint16_type_value ) )
     {
-#if 0
-        blas::gemv( traits::NO_TRANSPOSE,
-                    1.0, M_Bt[ q ], M_n[M_face_id],
-                    0.0, M_n_real );
-#else
         ublas::axpy_prod( M_B,
                           M_gm->referenceConvex().normal( M_face_id ),
                           M_n_real,
                           true );
-#endif
         M_n_norm = ublas::norm_2( M_n_real );
         M_u_n_real = M_n_real/M_n_norm;
 
@@ -1766,16 +1760,26 @@ void updateJKBN( mpl::bool_<true>  )
 
     if ( vm::has_tangent<context>::value && ( M_face_id != invalid_uint16_type_value ) )
     {
-        // t = |\hat{e}|*o_K*(K*t_ref)/|e| where o_K is the sign(e*x_K(\hat{e}))
-        ublas::axpy_prod( M_K,
-                          M_gm->referenceConvex().tangent( M_face_id ),
-                          M_t_real,
-                          true );
-        double ratio = M_gm->referenceConvex().h( M_face_id )/M_h_face;
+        if ( NDim == 2 )
+        {
+            // t = |\hat{e}|*o_K*(K*t_ref)/|e| where o_K is the sign(e*x_K(\hat{e}))
+            ublas::axpy_prod( M_K,
+                              M_gm->referenceConvex().tangent( M_face_id ),
+                              M_t_real,
+                              true );
+            double ratio = M_gm->referenceConvex().h( M_face_id )/M_h_face;
 
-        M_t_real *= ratio;
+            M_t_real *= ratio;
+        }
+
+        // compute projector on tangent plane
+        Eigen::Map<Eigen::Matrix<value_type,Eigen::Dynamic,Eigen::Dynamic,Eigen::RowMajor> > P(M_Ptangent.data().begin(), NDim, NDim);
+        P.setIdentity(NDim, NDim);
+        Eigen::Map<Eigen::Matrix<value_type,Eigen::Dynamic,Eigen::Dynamic,Eigen::RowMajor> > N(M_u_n_real.data().begin(), NDim,1);
+        P.noalias() -= N*N.transpose();
     }
 
+
 }
 
 /**
@@ -1783,6 +1787,9 @@ void updateJKBN( mpl::bool_<true>  )
  */
 void updateJKBN( mpl::bool_<false>  )
 {
+
+     const bool doComputeNormal = ( ( NDim != PDim ) || ( vm::has_normal<context>::value ) ) && ( M_face_id != invalid_uint16_type_value );
+
     //std::cout << "nPoints() =" << nPoints() << "\n";
     //VLOG(1) << "[geomap] G = "<< M_G << "\n";
     //double res = 0;
@@ -1844,7 +1851,7 @@ void updateJKBN( mpl::bool_<false>  )
 
         M_Jt[q] = M_J;
 
-        if ( vm::has_kb<context>::value )
+        if ( vm::has_kb<context>::value || doComputeNormal )
         {
             //VLOG(1) << "[geomap] B[" << q << "]= "<< M_B << "\n";
             M_Bt[q].resize( M_B.size1(), M_B.size2() );
@@ -1854,8 +1861,9 @@ void updateJKBN( mpl::bool_<false>  )
 
     }
 
+
     //VLOG(1) << "[geomap] res(sum J) = " << res << "\n";
-    if ( ( ( NDim != PDim ) || ( vm::has_normal<context>::value ) ) && ( M_face_id != invalid_uint16_type_value ) )
+    if ( doComputeNormal )
     {
         //std::cout << "has normal\n";
         for ( int q = 0; q < nPoints(); ++q )
@@ -1945,6 +1953,8 @@ matrix_type M_K;
 matrix_type M_CS;
 matrix_type M_CSi;
 matrix_type M_B;
+matrix_type M_Ptangent;
+
 boost::multi_array<value_type,4> M_B3;
 
 size_type M_id;
@@ -1970,6 +1980,8 @@ permutation_type M_perm;
 }; // Context
 
 
+
+
 template<size_type context_v, typename ElementType>
 boost::shared_ptr<Context<context_v,ElementType> >
 context( geometric_mapping_ptrtype gm, ElementType const& e, precompute_ptrtype const& pc )
@@ -1985,9 +1997,12 @@ boost::shared_ptr<Context<context_v,ElementType> >
 context( ElementType const& e, precompute_ptrtype const& pc )
 {
     return boost::shared_ptr<Context<context_v,ElementType> >(
-               new Context<context_v, ElementType>( this->shared_from_this(),
-                       e,
-                       pc ) );
+                new Context<context_v, ElementType>
+                (
+                 //super_enable_this::shared_from_this(),
+                 boost::dynamic_pointer_cast<GeoMap<Dim, Order, RealDim, T, Entity, PP > >(this->shared_from_this()),
+                e,
+                pc ) );
 }
 
 template<size_type context_v, typename ElementType>
@@ -2010,10 +2025,13 @@ context( ElementType const& e,
          uint16_type f )
 {
     return boost::shared_ptr<Context<context_v,ElementType> >(
-               new Context<context_v, ElementType>( this->shared_from_this(),
-                       e,
-                       pc,
-                       f ) );
+               new Context<context_v, ElementType>
+               (
+                //super_enable_this::shared_from_this(),
+                boost::dynamic_pointer_cast<GeoMap<Dim, Order, RealDim, T, Entity, PP > >(this->shared_from_this()),
+               e,
+               pc,
+               f ) );
 }
 
 
