@@ -57,6 +57,7 @@
 #include <feel/feelpoly/functionals2.hpp>
 #include <feel/feelpoly/quadpoint.hpp>
 #include <feel/feelpoly/fe.hpp>
+#include <feel/feelpoly/hdivpolynomialset.hpp>
 
 #include <feel/feelvf/vf.hpp>
 
@@ -92,6 +93,7 @@ struct times_x
     int M_c;
 };
 
+#if 0
 template< class T >
 struct extract_all_poly_indices
 {
@@ -106,25 +108,28 @@ struct extract_all_poly_indices
         return start++;
     }
 };
+#endif
+
 }// detail
 
 template<uint16_type N,
          uint16_type O,
          typename T = double,
-         template<uint16_type, uint16_type, uint16_type> class Convex = Simplex>
+         template<uint16_type, uint16_type, uint16_type> class Convex = Simplex,
+         uint16_type TheTAG = 0>
 class RaviartThomasPolynomialSet
     :
-    public Feel::detail::OrthonormalPolynomialSet<N, N, O+1, Vectorial, T, Convex>
+    public Feel::detail::OrthonormalPolynomialSet<N, O+1, N, Vectorial, T, TheTAG, Convex>
 {
-    typedef Feel::detail::OrthonormalPolynomialSet<N, N, O+1, Vectorial, T, Convex> super;
+    typedef Feel::detail::OrthonormalPolynomialSet<N, O+1, N, Vectorial, T, TheTAG, Convex> super;
 
 public:
     static const uint16_type Om1 = (O==0)?0:O-1;
-    typedef Feel::detail::OrthonormalPolynomialSet<N, N, O, Vectorial, T, Convex> Pk_v_type;
-    typedef Feel::detail::OrthonormalPolynomialSet<N, N, O+1, Vectorial, T, Convex> Pkp1_v_type;
-    typedef Feel::detail::OrthonormalPolynomialSet<N, N, Om1, Vectorial, T, Convex> Pkm1_v_type;
-    typedef Feel::detail::OrthonormalPolynomialSet<N, N, O, Scalar, T, Convex> Pk_s_type;
-    typedef Feel::detail::OrthonormalPolynomialSet<N, N, O+1, Scalar, T, Convex> Pkp1_s_type;
+    typedef Feel::detail::OrthonormalPolynomialSet<N, O, N, Vectorial, T, TheTAG, Convex> Pk_v_type;
+    typedef Feel::detail::OrthonormalPolynomialSet<N, O+1, N, Vectorial, T, TheTAG, Convex> Pkp1_v_type;
+    typedef Feel::detail::OrthonormalPolynomialSet<N, Om1, N, Vectorial, T, TheTAG, Convex> Pkm1_v_type;
+    typedef Feel::detail::OrthonormalPolynomialSet<N, O, N, Scalar, T, TheTAG, Convex> Pk_s_type;
+    typedef Feel::detail::OrthonormalPolynomialSet<N, O+1, N, Scalar, T, TheTAG, Convex> Pkp1_s_type;
 
     typedef PolynomialSet<typename super::basis_type,Vectorial> vectorial_polynomialset_type;
     typedef typename vectorial_polynomialset_type::polynomial_type vectorial_polynomial_type;
@@ -234,10 +239,15 @@ public:
     typedef PointSetType<convex_type, nOrder, value_type> pointset_type;
 
     static const uint16_type nbPtsPerVertex = 0;
-    static const uint16_type nbPtsPerEdge = mpl::if_<mpl::equal_to<mpl::int_<nDim>,mpl::int_<2> >,mpl::int_<reference_convex_type::nbPtsPerEdge>,mpl::int_<0> >::type::value;
-    static const uint16_type nbPtsPerFace =mpl::if_<mpl::equal_to<mpl::int_<nDim>,mpl::int_<3> >,mpl::int_<reference_convex_type::nbPtsPerFace>,mpl::int_<0> >::type::value;
-    static const uint16_type nbPtsPerVolume = 0;
-    static const uint16_type numPoints = ( reference_convex_type::numGeometricFaces*nbPtsPerFace+reference_convex_type::numEdges*nbPtsPerEdge );
+    static constexpr uint16_type nbPtsPerEdge = (nDim==2)?reference_convex_type::nbPtsPerEdge:0;
+    static constexpr uint16_type nbPtsPerFace2d = (nOrder)*(nOrder+2)-reference_convex_type::numEdges*nbPtsPerEdge;
+    static constexpr uint16_type nbPtsPerFace3d = reference_convex_type::nbPtsPerFace;
+    static constexpr uint16_type nbPtsPerFace =(nDim==3)?nbPtsPerFace3d:nbPtsPerFace2d;
+    static constexpr uint16_type nbInteriorMoments3d = (nOrder)*(nOrder+1)*(nOrder+3)/2-reference_convex_type::numTopologicalFaces*nbPtsPerFace3d;
+    static constexpr uint16_type nbPtsPerVolume = (nDim==3)?nbInteriorMoments3d:0;
+    static constexpr uint16_type numPoints2d = (nOrder)*(nOrder+2);
+    static constexpr uint16_type numPoints3d = (nOrder)*(nOrder+1)*(nOrder+3)/2;
+    static constexpr uint16_type numPoints = (nDim==2)?numPoints2d:numPoints3d;
 
     /** Number of degrees of freedom per vertex */
     static const uint16_type nDofPerVertex = 0;
@@ -249,7 +259,7 @@ public:
     static const uint16_type nDofPerFace = nbPtsPerFace;
 
     /** Number of degrees  of freedom per volume */
-    static const uint16_type nDofPerVolume = 0;
+    static const uint16_type nDofPerVolume = nbPtsPerVolume;
 
     /** Total number of degrees of freedom (equal to refEle::nDof) */
     static const uint16_type nLocalDof = numPoints;
@@ -344,6 +354,11 @@ public:
 
             //std::cout << "Pkm1 = " << Pkm1.coeff() << "\n";
             //std::cout << "Primal = " << primal.coeff() << "\n";
+            if ( nDim == 2 )
+                CHECK( Pkm1.polynomialDimension() == nbPtsPerFace )
+                    << "Invalid number of interior moments in 2D "
+                    << " Pkm1.dim = " << Pkm1.polynomialDimension()
+                    << " nbPtsPerFace = " << nbPtsPerFace;
             for ( int i = 0; i < Pkm1.polynomialDimension(); ++i )
             {
                 typedef functional::IntegralMoment<primal_space_type, vectorial_polynomialset_type> fim_type;
@@ -430,6 +445,7 @@ class RaviartThomas
 public FiniteElement<RaviartThomasPolynomialSet<N, O, T, Convex>,
     fem::detail::RaviartThomasDual,
     PointSetEquiSpaced >,
+public HDivPolynomialSet,
 public boost::enable_shared_from_this<RaviartThomas<N,O,T,Convex> >
 {
     typedef FiniteElement<RaviartThomasPolynomialSet<N, O, T, Convex>,
@@ -474,15 +490,10 @@ public:
 
     static const uint16_type nOrder =  dual_space_type::nOrder;
     static const uint16_type nbPtsPerVertex = 0;
-    static const uint16_type nbPtsPerEdge = mpl::if_<mpl::equal_to<mpl::int_<nDim>,mpl::int_<2> >,
-                             mpl::int_<reference_convex_type::nbPtsPerEdge>,
-                             mpl::int_<0> >::type::value;
-    static const uint16_type nbPtsPerFace = mpl::if_<mpl::equal_to<mpl::int_<nDim>,mpl::int_<3> >,
-                             mpl::int_<reference_convex_type::nbPtsPerFace>,
-                             mpl::int_<0> >::type::value;
-    static const uint16_type nbPtsPerVolume = 0;
-    static const uint16_type numPoints = ( reference_convex_type::numGeometricFaces*nbPtsPerFace+
-                                           reference_convex_type::numEdges*nbPtsPerEdge );
+    static const uint16_type nbPtsPerEdge = dual_space_type::nbPtsPerEdge;
+    static const uint16_type nbPtsPerFace = dual_space_type::nbPtsPerFace;
+    static const uint16_type nbPtsPerVolume = dual_space_type::nbPtsPerVolume;
+    static const uint16_type numPoints = dual_space_type::numPoints;
 
     static const uint16_type nLocalDof = dual_space_type::nLocalDof;
     //@}
@@ -509,6 +520,25 @@ public:
         std::cout << "[RT] is_product : " << is_product << "\n";
 #endif
     }
+
+    template<int subN>
+    struct SubSpace
+    {
+        typedef RaviartThomas<N-1, O, T, Convex, TheTAG> type;
+    };
+
+    struct SSpace
+    {
+        typedef  RaviartThomas<N, O, T, Convex, TheTAG> type;
+
+    };
+
+    template<uint16_type NewDim>
+    struct ChangeDim
+    {
+        typedef  RaviartThomas<NewDim, O, T, Convex,  TheTAG> type;
+    };
+
     RaviartThomas( RaviartThomas const & cr )
         :
         super( cr ),
@@ -558,12 +588,12 @@ public:
 
     template<typename ExprType>
     static auto
-    isomorphism( ExprType expr ) -> decltype( Feel::vf::detJ()*( trans( Feel::vf::JinvT() )*expr )*Feel::vf::Nref() )
-    //isomorphism( ExprType& expr ) -> decltype( expr )
+    //isomorphism( ExprType expr ) -> decltype( Feel::vf::detJ()*( trans( Feel::vf::JinvT() )*expr )*Feel::vf::Nref() )
+    isomorphism( ExprType& expr ) -> decltype( expr )
     {
         using namespace Feel::vf;
-        return detJ()*( trans( JinvT() )*expr )*Nref();
-        //return expr;
+        //return detJ()*( trans( JinvT() )*expr )*Nref();
+        return expr;
     }
 #if 0
     /**
@@ -579,7 +609,7 @@ public:
         typedef fusion::map<fusion::pair<vf::detail::gmc<0>, gmc_ptrtype> > map_gmc_type;
 
         std::vector<value_type> v( nLocalDof );
-
+        auto I = integrate( face, _expr=_e1*N() );
         // First deal with the face dof
         for ( int face = 0; face < numTopologicalFaces; ++face )
         {
