@@ -7,6 +7,7 @@
 
   Copyright (C) 2006 EPFL
   Copyright (C) 2007-2010 Université Joseph Fourier (Grenoble I)
+  Copyright (C) 2010-2014 Feel++ Consortium
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -43,10 +44,12 @@
 # include <boost/preprocessor/punctuation/comma.hpp>
 # include <boost/preprocessor/facilities/identity.hpp>
 
-namespace Feel
-{
-namespace vf
-{
+
+#include <feel/feelvf/expr.hpp>
+
+
+namespace Feel { namespace vf {
+
 /// \cond detail
 # /* Information about C operators */
 #
@@ -71,7 +74,7 @@ const size_type jkp = vm::KB|vm::JACOBIAN|vm::POINT;
 #if 1
 # define VF_GD                                                          \
    BOOST_PP_TUPLE_TO_LIST(                                              \
-       21,                                                              \
+       22,                                                              \
       (                                                                 \
        ( N       , GDN       , 0, jkbn, Vectorial, M_gmc->unitNormal( q )[ c1 ] , 0), \
        ( Nx      , GDNx      , 0, jkbn, Scalar   , M_gmc->unitNormal( q )[ 0 ]  , 0), \
@@ -79,6 +82,7 @@ const size_type jkp = vm::KB|vm::JACOBIAN|vm::POINT;
        ( Nz      , GDNz      , 2, jkbn, Scalar   , M_gmc->unitNormal( q )[ 2 ]  , 0), \
        ( Nref    , GDNref    , 0, 0, Vectorial, M_gmc->refNormal( q )[ c1 ] , 0), \
        ( normalNorm, GDnormalNorm, 0, 0, Scalar, M_gmc->normalNorm( q ) , 0), \
+       ( pT      , GDpT      , 0, jt, Tensor2, M_gmc->projectorTangent( c1, c2, q), 0), \
        ( T       , GDT       , 0, jt, Vectorial, M_gmc->tangent( q )[ c1 ], 0), \
        ( Tx      , GDTx      , 0, jt, Scalar   , M_gmc->tangent( q )[ 0 ] , 0), \
        ( Ty      , GDTy      , 1, jt, Scalar   , M_gmc->tangent( q )[ 1 ] , 0), \
@@ -174,6 +178,7 @@ const size_type jkp = vm::KB|vm::JACOBIAN|vm::POINT;
         };                                                              \
         typedef VF_GD_NAME(O) this_type;                                \
         typedef double value_type;                                      \
+        typedef value_type evaluate_type;                               \
                                                                         \
         VF_GD_NAME(O) ()                                                \
         {                                                               \
@@ -181,14 +186,14 @@ const size_type jkp = vm::KB|vm::JACOBIAN|vm::POINT;
         VF_GD_NAME(O) ( VF_GD_NAME(O) const& /*__vf*/ )                 \
         {                                                               \
         }                                                               \
-        template<typename TheExpr>                                      \
+        template<typename... TheExpr>                                      \
         struct Lambda                                                   \
         {                                                               \
             typedef this_type type;                                     \
         };                                                              \
-        template<typename TheExpr>                                      \
-        typename Lambda<TheExpr>::type                                  \
-        operator()( TheExpr const& e  ) { return this_type(); }         \
+        template<typename... TheExpr>                                      \
+        typename Lambda<TheExpr...>::type                                  \
+        operator()( TheExpr... e  ) { return this_type(); }         \
                                                                         \
                                                                         \
         template<typename Geo_t, typename Basis_i_t, typename Basis_j_t = Basis_i_t> \
@@ -201,6 +206,7 @@ const size_type jkp = vm::KB|vm::JACOBIAN|vm::POINT;
             typedef typename fusion::result_of::value_at_key<Geo_t,key_type>::type::element_type* gmc_ptrtype; \
             typedef typename fusion::result_of::value_at_key<Geo_t,key_type>::type::element_type gmc_type; \
             typedef typename gmc_type::value_type value_type;           \
+            typedef  value_type evaluate_type;                          \
             typedef VF_GD_RETURN(O)<gmc_type::NDim> return_value_type;  \
             typedef Shape<gmc_type::NDim, VF_GD_RETURN(O), false> shape; \
                                                                         \
@@ -243,6 +249,11 @@ const size_type jkp = vm::KB|vm::JACOBIAN|vm::POINT;
             {                                                           \
                 /*BOOST_STATIC_ASSERT( dim_ok );*/                      \
                 update( geom );                                         \
+            }                                                           \
+            template<typename CTX>                                      \
+                void updateContext( CTX const& ctx )                    \
+            {                                                           \
+                M_gmc = ctx->gmContext().get();                        \
             }                                                           \
                                                                         \
                 value_type                                              \
