@@ -210,9 +210,12 @@ enum OperatorType { __TEST, __TRIAL, __VALUE };
             typedef typename element_type::functionspace_type functionspace_type; \
             typedef typename functionspace_type::reference_element_type* fe_ptrtype; \
             typedef typename functionspace_type::reference_element_type fe_type; \
+            typedef typename functionspace_type::mortar_fe_type mortar_fe_type; \
             typedef typename functionspace_type::geoelement_type geoelement_type; \
             typedef typename functionspace_type::gm_type gm_type; \
             typedef typename functionspace_type::value_type value_type; \
+            typedef value_type evaluate_type;                           \
+                                                                        \
             static const uint16_type rank = fe_type::rank;              \
             static const uint16_type nComponents1 = fe_type::nComponents1; \
             static const uint16_type nComponents2 = fe_type::nComponents2; \
@@ -225,38 +228,44 @@ enum OperatorType { __TEST, __TRIAL, __VALUE };
             template<typename Func>                                     \
                 struct HasTestFunction                                  \
             {                                                           \
-                static const bool result = VF_OP_SWITCH( BOOST_PP_OR( VF_OP_TYPE_IS_TRIAL( T ), VF_OP_TYPE_IS_VALUE( T ) ), false , (boost::is_same<Func,fe_type>::value) ); \
+                static const bool result = VF_OP_SWITCH( BOOST_PP_OR( VF_OP_TYPE_IS_TRIAL( T ), VF_OP_TYPE_IS_VALUE( T ) ), false , \
+                                                         (boost::is_same<Func,fe_type>::value||(element_type::is_mortar&&boost::is_same<Func,mortar_fe_type>::value)) ); \
             };                                                          \
                                                                         \
             template<typename Func>                                     \
                 struct HasTrialFunction                                 \
             {                                                           \
-                static const bool result = VF_OP_SWITCH( VF_OP_TYPE_IS_TRIAL( T ), (boost::is_same<Func,fe_type>::value), false ); \
+                static const bool result = VF_OP_SWITCH( VF_OP_TYPE_IS_TRIAL( T ), \
+                                                         (boost::is_same<Func,fe_type>::value||(element_type::is_mortar&&boost::is_same<Func,mortar_fe_type>::value)), false ); \
             };                                                          \
                                                                         \
                                                                         \
-            VF_OPERATOR_NAME( O ) ( element_type const& v )             \
-                : M_v ( boost::cref(v) )                               \
+            VF_OPERATOR_NAME( O ) ( element_type const& v, bool useInterpWithConfLoc=false ) \
+              : M_v ( boost::cref(v) ),                                 \
+              M_useInterpWithConfLoc( useInterpWithConfLoc )            \
             {                                                           \
                 if ( VF_OP_TYPE_IS_VALUE( T ) )                         \
                     v.updateGlobalValues();                             \
                 DVLOG(2) << "[" BOOST_PP_STRINGIZE(VF_OPERATOR_NAME( O )) "] default constructor\n"; \
             }                                                           \
             VF_OPERATOR_NAME( O )( VF_OPERATOR_NAME( O ) const& op )    \
-                : M_v ( op.M_v )                                      \
+              : M_v ( op.M_v ),                                         \
+              M_useInterpWithConfLoc( op.M_useInterpWithConfLoc )       \
+                                                                        \
             {                                                           \
                 DVLOG(2) << "[" BOOST_PP_STRINGIZE(VF_OPERATOR_NAME( O )) "] copy constructor\n"; \
             }                                                           \
-            template<typename TheExpr>                                  \
+            template<typename... TheExpr>                               \
             struct Lambda                                               \
             {                                                           \
                 typedef this_type type;                                 \
             };                                                          \
-            template<typename TheExpr>                                  \
-                typename Lambda<TheExpr>::type                          \
-                operator()( TheExpr const& e  ) { return *this; }       \
+            template<typename... TheExpr>                               \
+                typename Lambda<TheExpr...>::type                       \
+                operator()( TheExpr... e) { return *this; }             \
                                                                         \
             element_type const& e() const { return M_v; }              \
+            bool useInterpWithConfLoc() const { return M_useInterpWithConfLoc; } \
             template<typename Geo_t, typename Basis_i_t, typename Basis_j_t = Basis_i_t> \
                 struct tensor                                           \
             {                                                           \
@@ -313,8 +322,10 @@ enum OperatorType { __TEST, __TRIAL, __VALUE };
                 };                                                      \
                 static const bool dim_ok  = (VF_OPERATOR_TYPE_COMP(O) < gmc_type::NDim); \
                 static const bool fe_ok  = mpl::if_<mpl::bool_<VF_OP_TYPE_IS_VALUE( T )>, \
-                    mpl::bool_<true>,                                   \
-                    boost::is_same<typename ttt<basis_context_type>::type::reference_element_type, fe_type> >::type::value; \
+                                                    mpl::bool_<true>,   \
+                                                    typename mpl::or_<boost::is_same<typename ttt<basis_context_type>::type::reference_element_type, fe_type>, \
+                                                                      mpl::and_<mpl::bool_<element_type::is_mortar>,boost::is_same<typename ttt<basis_context_type>::type::reference_element_type, mortar_fe_type> > \
+                                                                      >::type >::type::value; \
                 struct is_zero {                                        \
                     /*static const bool value = !(dim_ok && fe_ok);*/   \
                     static const bool value = false;                    \
@@ -337,6 +348,7 @@ enum OperatorType { __TEST, __TRIAL, __VALUE };
                     M_loc(VF_OP_SWITCH_ELSE_EMPTY( VF_OP_TYPE_IS_VALUE( T ), M_expr.e().BOOST_PP_CAT(VF_OPERATOR_TERM( O ),Extents)(*M_geot) ) ), \
                     M_zero( ret_type::Zero() ),                         \
                     M_did_init( t.M_did_init ),                         \
+                    M_hasRelationMesh( t.M_hasRelationMesh ),           \
                     M_same_mesh( t.M_same_mesh )                        \
                         {                                               \
                         }                                               \
@@ -360,12 +372,12 @@ enum OperatorType { __TEST, __TRIAL, __VALUE };
                     M_loc(VF_OP_SWITCH_ELSE_EMPTY( VF_OP_TYPE_IS_VALUE( T ), expr.e().BOOST_PP_CAT(VF_OPERATOR_TERM( O ),Extents)(*fusion::at_key<key_type>( geom )) ) ), \
                     M_zero( ret_type::Zero() ),                         \
                     M_did_init( false ),                                \
-                    M_same_mesh( fusion::at_key<key_type>( geom )->element().mesh()->isRelatedTo( expr.e().functionSpace()->mesh()) && isSameGeo ) \
+                    M_hasRelationMesh( fusion::at_key<key_type>( geom )->element().mesh()->isRelatedTo( expr.e().functionSpace()->mesh()) ), \
+                    M_same_mesh( M_hasRelationMesh && isSameGeo )         \
                         {                                               \
                             if(!M_same_mesh)                            \
-                                expr.e().functionSpace()->mesh()->tool_localization()->updateForUse(); \
-                            /*update( geom );*/                         \
-                    }                                                   \
+                                    expr.e().functionSpace()->mesh()->tool_localization()->updateForUse(); \
+                        }                                               \
                 tensor( this_type const& expr,                          \
                         Geo_t const& geom,                              \
                         Basis_i_t const& VF_OP_SWITCH_ELSE_EMPTY( VF_OP_TYPE_IS_TEST( T ), fev  ) ) \
@@ -382,7 +394,8 @@ enum OperatorType { __TEST, __TRIAL, __VALUE };
                     M_loc(VF_OP_SWITCH_ELSE_EMPTY( VF_OP_TYPE_IS_VALUE( T ), expr.e().BOOST_PP_CAT(VF_OPERATOR_TERM( O ),Extents)(*fusion::at_key<key_type>( geom )) ) ), \
                     M_zero( ret_type::Zero() ),                         \
                     M_did_init( false ),                                \
-                    M_same_mesh( fusion::at_key<key_type>( geom )->element().mesh()->isRelatedTo( expr.e().functionSpace()->mesh()) && isSameGeo ) \
+                    M_hasRelationMesh( fusion::at_key<key_type>( geom )->element().mesh()->isRelatedTo( expr.e().functionSpace()->mesh()) ), \
+                    M_same_mesh( M_hasRelationMesh && isSameGeo )         \
                         {                                               \
                             if(!M_same_mesh)                            \
                                 expr.e().functionSpace()->mesh()->tool_localization()->updateForUse(); \
@@ -401,7 +414,8 @@ enum OperatorType { __TEST, __TRIAL, __VALUE };
                     M_loc(VF_OP_SWITCH_ELSE_EMPTY( VF_OP_TYPE_IS_VALUE( T ), expr.e().BOOST_PP_CAT(VF_OPERATOR_TERM( O ),Extents)(*fusion::at_key<key_type>( geom )) ) ), \
                     M_zero( ret_type::Zero() ),                         \
                     M_did_init( false ),                                \
-                    M_same_mesh( fusion::at_key<key_type>( geom )->element().mesh()->isRelatedTo( expr.e().functionSpace()->mesh()) && isSameGeo ) \
+                    M_hasRelationMesh( fusion::at_key<key_type>( geom )->element().mesh()->isRelatedTo( expr.e().functionSpace()->mesh()) ), \
+                    M_same_mesh( M_hasRelationMesh && isSameGeo )         \
                         {                                               \
                             if(!M_same_mesh)                            \
                                 expr.e().functionSpace()->mesh()->tool_localization()->updateForUse(); \
@@ -481,6 +495,12 @@ enum OperatorType { __TEST, __TRIAL, __VALUE };
                     VF_OP_SWITCH_ELSE_EMPTY( VF_OP_TYPE_IS_TEST( T ),   \
                                              M_fec = fusion::at_key<basis_context_key_type>( fev ).get() ) ; \
                 }                                                       \
+                template <typename CTX>                                 \
+                    void updateContext( CTX const& ctx )                \
+                {                                                       \
+                    std::fill( M_loc.data(), M_loc.data()+M_loc.num_elements(), loc_type::Zero() ); \
+                    M_expr.e().VF_OPERATOR_SYMBOL( O )( *ctx, M_loc ); \
+                }                                                       \
                 void update( Geo_t const& geom )                        \
                 {                                                       \
                     /*BOOST_STATIC_ASSERT( dim_ok );*/                  \
@@ -499,7 +519,8 @@ enum OperatorType { __TEST, __TRIAL, __VALUE };
                         M_expr.e().VF_OPERATOR_SYMBOL( O )( *M_ctx, M_loc ); \
                     else  {                                             \
                         matrix_node_type __ptsreal = M_expr.e().ptsInContext(*fusion::at_key<key_type>( geom ), mpl::int_<2>()); \
-                        M_expr.e().BOOST_PP_CAT(VF_OPERATOR_SYMBOL( O ),Interpolate)( /**M_ctx,*/ __ptsreal, M_loc ); \
+                        M_expr.e().BOOST_PP_CAT(VF_OPERATOR_SYMBOL( O ),Interpolate)( /**M_ctx,*/ __ptsreal, M_loc, M_expr.useInterpWithConfLoc()/*false*//*true*/, \
+                                                                                      fusion::at_key<key_type>( geom )->element().face( fusion::at_key<key_type>( geom )->faceId() ).vertices() ); \
                     }                                                   \
                 }                                                       \
                 void update( Geo_t const& geom, mpl::bool_<true> )      \
@@ -513,7 +534,10 @@ enum OperatorType { __TEST, __TRIAL, __VALUE };
                     else {                                              \
                         /*std::cout << "\n idv with interp \n";*/       \
                         matrix_node_type __ptsreal = M_expr.e().ptsInContext(*fusion::at_key<key_type>( geom ), mpl::int_<1>()); \
-                        M_expr.e().BOOST_PP_CAT(VF_OPERATOR_SYMBOL( O ),Interpolate)( /**M_ctx,*/ __ptsreal, M_loc ); \
+                        auto setOfPts = ( fusion::at_key<key_type>( geom )->faceId() != invalid_uint16_type_value )? \
+                          fusion::at_key<key_type>( geom )->element().face( fusion::at_key<key_type>( geom )->faceId() ).vertices() : \
+                          fusion::at_key<key_type>( geom )->element().vertices(); \
+                        M_expr.e().BOOST_PP_CAT(VF_OPERATOR_SYMBOL( O ),Interpolate)( /**M_ctx,*/ __ptsreal, M_loc, M_expr.useInterpWithConfLoc()/*false*//*true*/, setOfPts  ); \
                     }                                                   \
                 }                                                       \
                 void update( Geo_t const& geom, mpl::bool_<false> )     \
@@ -701,31 +725,33 @@ enum OperatorType { __TEST, __TRIAL, __VALUE };
                 ret_type M_zero;                                        \
                 /*typename element_type::BOOST_PP_CAT( VF_OPERATOR_TERM( O ), _type) M_loc;*/ \
                 bool M_did_init;                                        \
+                const bool M_hasRelationMesh;                           \
                 const bool M_same_mesh;                                 \
             };                                                          \
                                                                         \
         protected:                                                      \
             VF_OPERATOR_NAME( O ) () {}                                 \
             boost::reference_wrapper<const element_type>  M_v;         \
+            bool M_useInterpWithConfLoc;                                     \
         };                                                              \
     template <class ELEM                                                \
               BOOST_PP_IF( VF_OP_TYPE_IS_GENERIC( T ),  BOOST_PP_COMMA, BOOST_PP_EMPTY )() \
         BOOST_PP_IF( VF_OP_TYPE_IS_GENERIC( T ),  BOOST_PP_IDENTITY( VF_OP_TYPE_TYPE( T ) sw ), BOOST_PP_EMPTY )() > \
     inline Expr< VF_OPERATOR_NAME( O )< ELEM, VF_OP_TYPE_OBJECT(T)> >   \
-    BOOST_PP_CAT( VF_OPERATOR_SYMBOL(O), VF_OP_TYPE_SUFFIX(T) )( ELEM const& expr ) \
+    BOOST_PP_CAT( VF_OPERATOR_SYMBOL(O), VF_OP_TYPE_SUFFIX(T) )( ELEM const& expr,bool useInterpWithConfLoc=false ) \
         {                                                               \
             typedef VF_OPERATOR_NAME( O )< ELEM, VF_OP_TYPE_OBJECT(T)> expr_t; \
-            return Expr< expr_t >(  expr_t(expr) );                     \
+            return Expr< expr_t >(  expr_t(expr,useInterpWithConfLoc) ); \
         }                                                               \
                                                                         \
     template <class ELEM                                                \
               BOOST_PP_IF( VF_OP_TYPE_IS_GENERIC( T ),  BOOST_PP_COMMA, BOOST_PP_EMPTY )() \
         BOOST_PP_IF( VF_OP_TYPE_IS_GENERIC( T ),  BOOST_PP_IDENTITY( VF_OP_TYPE_TYPE( T ) sw ), BOOST_PP_EMPTY )() > \
     inline Expr< VF_OPERATOR_NAME( O )< ELEM, VF_OP_TYPE_OBJECT(T)> >   \
-    BOOST_PP_CAT( VF_OPERATOR_SYMBOL(O), VF_OP_TYPE_SUFFIX(T) )( boost::shared_ptr<ELEM> expr ) \
+    BOOST_PP_CAT( VF_OPERATOR_SYMBOL(O), VF_OP_TYPE_SUFFIX(T) )( boost::shared_ptr<ELEM> expr,bool useInterpWithConfLoc=false ) \
         {                                                               \
             typedef VF_OPERATOR_NAME( O )< ELEM, VF_OP_TYPE_OBJECT(T)> expr_t; \
-            return Expr< expr_t >(  expr_t(*expr) );                     \
+            return Expr< expr_t >(  expr_t(*expr,useInterpWithConfLoc) ); \
         }                                                               \
 
 /**/
